/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.client;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.common.TeamBuildDuplicateItemException;
import com.ibm.team.build.common.TeamBuildStateException;
import com.ibm.team.build.common.model.IBuildCategoryHandle;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.common.model.IBuildFolder;
import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompileContributionHandle;
import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.common.model.ICompilePackageHandle;
import com.ibm.team.build.common.model.IJUnitTestClass;
import com.ibm.team.build.common.model.IJUnitTestClassHandle;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.common.model.IJUnitTestSuiteContributionHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.repository.common.query.IDataQuery;
import com.ibm.team.repository.common.query.IDataQueryPage;
import com.ibm.team.repository.common.query.IItemQuery;
import com.ibm.team.repository.common.query.IItemQueryPage;
import com.ibm.team.repository.common.query.IQueryPage;
import com.ibm.team.repository.common.service.IQueryService;

/**
 * Base interface for all team build client library interfaces. It provides
 * support for common functionality that can be executed through any of the team
 * build client libraries.
 * 
 * @since 0.6
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface ITeamBuildBaseClient {

    /**
     * Save the given build definition to the repository. If the build
     * definition is new, the <code>IBuildAverageData</code>item will also be
     * created and assigned to the build definition.
     * 
     * @param buildDefinition
     *            The build definition to save. Must be a working copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the build definition.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws TeamBuildDuplicateItemException
     *             If a build definition with the id of the definition being
     *             saved already exists.
     */
    public IBuildDefinition save(IBuildDefinition buildDefinition, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, TeamBuildDuplicateItemException, IllegalArgumentException;

    /**
     * Save the given junit test suite contribution to the repository.
     * 
     * @param junitTestSuiteContribution
     *            The junit test suite contribution to save. Must be a working
     *            copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the junit test suite contribution.
     * @throws IllegalArgumentException
     *             If the <code>junitTestSuiteContribution</code> is
     *             <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IJUnitTestSuiteContribution save(IJUnitTestSuiteContribution junitTestSuiteContribution,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Save the given junit test class to the repository.
     * 
     * @param testClass
     *            The junit test class to save. Must be a working copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the junit test class.
     * @throws IllegalArgumentException
     *             If the <code>testClass</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IJUnitTestClass save(IJUnitTestClass testClass, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Save the given compile contribution to the repository.
     * 
     * @param compileContribution
     *            The compile contribution to save. Must be a working copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the compile contribution.
     * @throws IllegalArgumentException
     *             If the <code>compileContribution</code> is
     *             <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public ICompileContribution save(ICompileContribution compileContribution, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Save the given compile package to the repository.
     * 
     * @param compilePackage
     *            The compile package to save. Must be a working copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the compile package.
     * @throws IllegalArgumentException
     *             If the <code>compilePackage</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public ICompilePackage save(ICompilePackage compilePackage, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Save the given build engine to the repository.
     * 
     * @param buildEngine
     *            The build engine to save. Must be a working copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the build engine.
     * @throws IllegalArgumentException
     *             If the <code>buildEngine</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws TeamBuildDuplicateItemException
     *             If a build engine with the id of the engine being
     *             saved already exists.
     */
    public IBuildEngine save(IBuildEngine buildEngine, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, TeamBuildDuplicateItemException, IllegalArgumentException;

    /**
     * Save the given build result to the repository.
     * 
     * @param buildResult
     *            The build result to save. Must be a working copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the build result.
     * @throws IllegalArgumentException
     *             If the <code>buildResult</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildResult save(IBuildResult buildResult, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;
    
    /**
     * Saves the given build folder to the repository. Build definition folders
     * with a <code>null</code> parent will be visible in the root Builds node
     * and build engine folders with a <code>null</code> parent will be visible
     * in the Build Engines node. If the parent is not <code>null</code> then it
     * must exist. Folder name need not be unique. Changing the project area of
     * an existing folder is not supported.
     * 
     * @param buildFolder
     *            The folder to save. Must be a working copy.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The new state of the build folder.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the <code>buildFolder</code> is <code>null</code>.
     */
    public IBuildFolder save(IBuildFolder buildFolder, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified build definition from the repository. The build
     * definition is permanently removed from the repository. The build
     * definition can not be deleted if there are any build results associated
     * with it in the repository. The build results must be deleted first.
     * 
     * @param buildDefinitionHandle
     *            The handle to the build definition to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     * @throws TeamBuildStateException
     *             If the repository contains any build results that are
     *             associated with the build definition being deleted.
     */
    public void delete(IBuildDefinitionHandle buildDefinitionHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, TeamBuildStateException, IllegalArgumentException;

    /**
     * Deletes the specified JUnit test suite from the repository. The JUnit
     * test suite is permanently removed from the repository.
     * 
     * @param junitTestSuiteContributionHandle
     *            The handle to the JUnit test suite to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>junitTestSuiteContribution</code> is
     *             <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     */
    public void delete(IJUnitTestSuiteContributionHandle junitTestSuiteContributionHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified JUnit test class from the repository. The JUnit
     * test class is permanently removed from the repository.
     * 
     * @param testClassHandle
     *            The handle to the JUnit test class to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>testClass</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     */
    public void delete(IJUnitTestClassHandle testClassHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified compile contribution from the repository. The
     * compile contribution is permanently removed from the repository.
     * 
     * @param compileContributionHandle
     *            The handle to the compile contribution to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>compileContribution</code> is
     *             <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     */
    public void delete(ICompileContributionHandle compileContributionHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified compile package from the repository. The compile
     * package is permanently removed from the repository.
     * 
     * @param compilePackageHandle
     *            The handle to the compile package to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>compilePackage</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     */
    public void delete(ICompilePackageHandle compilePackageHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified build engine from the repository. The build engine
     * is permanently removed from the repository.
     * 
     * @param buildEngineHandle
     *            The handle to the build engine to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>buildEngine</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     */
    public void delete(IBuildEngineHandle buildEngineHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified build result from the repository. The build result
     * is permanently removed from the repository.
     * 
     * @param buildResultHandle
     *            The handle to the build result to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>buildResult</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     * @throws TeamBuildStateException
     *             If an attempt is made to delete a build result that has been
     *             marked to not allow deletion.
     */
    public void delete(IBuildResultHandle buildResultHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, TeamBuildStateException, IllegalArgumentException;

    /**
     * Deletes the given build folders, their child folders and any associations
     * within the hierarchy. Build definitions and engines within the hierarchy
     * will be uncategorized after this operation.
     * 
     * @param buildFolderHandles
     *            Array of {@link IBuildFolderHandle}s to be deleted. Folders
     *            that do not exist within the repository will be ignored.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the <code>buildFolderHandles</code> is <code>null</code> or
     *             contains <code>null</code> elements.
     */
    public void delete(IBuildFolderHandle[] buildFolderHandles, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified build category from the repository. The build
     * category is permanently removed from the repository.
     * 
     * @param buildCategoryHandle
     *            The handle to the build category to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws IllegalArgumentException
     *             If the <code>buildCategory</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while deleting the item from the
     *             repository.
     */
    public void delete(IBuildCategoryHandle buildCategoryHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Thin wrapper around {@link IQueryService#fetchPage(UUID, int, int)} that
     * supports the ability to cancel.
     * 
     * @param token
     *            See {@link IQueryService#fetchPage(UUID, int, int)} for
     *            details.
     * @param startIndex
     *            See {@link IQueryService#fetchPage(UUID, int, int)} for
     *            details.
     * @param pageSize
     *            See {@link IQueryService#fetchPage(UUID, int, int)} for
     *            details.
     * @param progressMonitor
     *            Allows the fetch of the page to be canceled.
     * @return See {@link IQueryService#fetchPage(UUID, int, int)} for details.
     * @throws TeamRepositoryException
     *             See {@link IQueryService#fetchPage(UUID, int, int)} for
     *             details.
     */
    public IQueryPage fetchPage(final UUID token, final int startIndex, final int pageSize,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Thin wrapper around
     * {@link IQueryService#uncommittedQueryItems(IItemQuery, Object[], int)}
     * that supports the ability to cancel.
     * 
     * @param itemQuery
     *            See
     *            {@link IQueryService#uncommittedQueryItems(IItemQuery, Object[], int)}
     *            for details.
     * @param parameters
     *            See
     *            {@link IQueryService#uncommittedQueryItems(IItemQuery, Object[], int)}
     *            for details.
     * @param pageSize
     *            See
     *            {@link IQueryService#uncommittedQueryItems(IItemQuery, Object[], int)}
     *            for details.
     * @param progressMonitor
     *            Allows the query of the items to be canceled.
     * @return See
     *         {@link IQueryService#uncommittedQueryItems(IItemQuery, Object[], int)}
     *         for details.
     * @throws TeamRepositoryException
     *             See
     *             {@link IQueryService#uncommittedQueryItems(IItemQuery, Object[], int)}
     *             for details.
     */
    public IItemQueryPage uncommittedQueryItems(final IItemQuery itemQuery, final Object[] parameters,
            final int pageSize, final IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Thin wrapper around
     * {@link IQueryService#queryItems(IItemQuery, Object[], int)} that supports
     * the ability to cancel.
     * 
     * @param itemQuery
     *            See
     *            {@link IQueryService#queryItems(IItemQuery, Object[], int)}
     *            for details.
     * @param parameters
     *            See
     *            {@link IQueryService#queryItems(IItemQuery, Object[], int)}
     *            for details.
     * @param pageSize
     *            See
     *            {@link IQueryService#queryItems(IItemQuery, Object[], int)}
     *            for details.
     * @param progressMonitor
     *            Allows the query of the items to be canceled.
     * @return See {@link IQueryService#queryItems(IItemQuery, Object[], int)}
     *         for details.
     * @throws TeamRepositoryException
     *             See
     *             {@link IQueryService#queryItems(IItemQuery, Object[], int)}
     *             for details.
     */
    public IItemQueryPage queryItems(final IItemQuery itemQuery, final Object[] parameters, final int pageSize,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Thin wrapper around
     * {@link IQueryService#uncommittedQueryData(IDataQuery, Object[], int)}
     * that supports the ability to cancel.
     * 
     * @param dataQuery
     *            See
     *            {@link IQueryService#uncommittedQueryData(IDataQuery, Object[], int)}
     *            for details.
     * @param parameters
     *            See
     *            {@link IQueryService#uncommittedQueryData(IDataQuery, Object[], int)}
     *            for details.
     * @param pageSize
     *            See
     *            {@link IQueryService#uncommittedQueryData(IDataQuery, Object[], int)}
     *            for details.
     * @param progressMonitor
     *            Allows the query of the data to be canceled.
     * @return See
     *         {@link IQueryService#uncommittedQueryData(IDataQuery, Object[], int)}
     *         for details.
     * @throws TeamRepositoryException
     *             See
     *             {@link IQueryService#uncommittedQueryData(IDataQuery, Object[], int)}
     *             for details.
     */
    public IDataQueryPage uncommittedQueryData(final IDataQuery dataQuery, final Object[] parameters,
            final int pageSize, final IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Thin wrapper around
     * {@link IQueryService#queryData(IDataQuery, Object[], int)} that supports
     * the ability to cancel.
     * 
     * @param dataQuery
     *            See {@link IQueryService#queryData(IDataQuery, Object[], int)}
     *            for details.
     * @param parameters
     *            See {@link IQueryService#queryData(IDataQuery, Object[], int)}
     *            for details.
     * @param pageSize
     *            See {@link IQueryService#queryData(IDataQuery, Object[], int)}
     *            for details.
     * @param progressMonitor
     *            Allows the query of the data to be canceled.
     * @return See {@link IQueryService#queryData(IDataQuery, Object[], int)}
     *         for details.
     * @throws TeamRepositoryException
     *             See
     *             {@link IQueryService#queryData(IDataQuery, Object[], int)}
     *             for details.
     */
    public IDataQueryPage queryData(final IDataQuery dataQuery, final Object[] parameters, final int pageSize,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException;

}
