/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.client;

import java.sql.Timestamp;
import org.eclipse.core.runtime.IProgressMonitor;
import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.TeamBuildStateException;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildActivity;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.internal.common.model.dto.IBuildItemNamePair;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Team build interface - provides access to team build information.
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface ITeamBuildClient extends ITeamBuildBaseClient {

    /**
     * The java system property for the http proxy host setting.
     * 
     * @since 0.6.1
     */
    public static final String BUILD_CLIENT_HTTP_PROXY_HOST_PROPERTY = "http.proxyHost"; //$NON-NLS-1$
    
    /**
     * The java system property for the http proxy port setting.
     * 
     * @since 0.6.1
     */
    public static final String BUILD_CLIENT_HTTP_PROXY_PORT_PROPERTY = "http.proxyPort"; //$NON-NLS-1$

    /**
     * The default port setting for the http proxy.
     * 
     * @since 0.6.1
     */
    public static final String BUILD_CLIENT_HTTP_PROXY_PORT_DEFAULT = "80"; //$NON-NLS-1$
    
    /**
     * The java system property for the https proxy host setting.
     * 
     * @since 0.6.1
     */
    public static final String BUILD_CLIENT_HTTPS_PROXY_HOST_PROPERTY = "https.proxyHost"; //$NON-NLS-1$
    
    /**
     * The java system property for the https proxy port setting.
     * 
     * @since 0.6.1
     */
    public static final String BUILD_CLIENT_HTTPS_PROXY_PORT_PROPERTY = "https.proxyPort"; //$NON-NLS-1$

    /**
     * The default port setting for the https proxy.
     * 
     * @since 0.6.1
     */
    public static final String BUILD_CLIENT_HTTPS_PROXY_PORT_DEFAULT = "443"; //$NON-NLS-1$

    /**
     * The java system property for the http proxy user.
     * 
     * @since 0.8.1
     */
    public static final String BUILD_CLIENT_HTTP_PROXY_USER_PROPERTY = "http.proxyUser"; //$NON-NLS-1$

    /**
     * The java system property for the http proxy password.
     * 
     * @since 0.8.1
     */
    public static final String BUILD_CLIENT_HTTP_PROXY_PASSWORD_PROPERTY = "http.proxyPassword"; //$NON-NLS-1$

    /**
     * The java system property for the https proxy user.
     * 
     * @since 0.8.1
     */
    public static final String BUILD_CLIENT_HTTPS_PROXY_USER_PROPERTY = "https.proxyUser"; //$NON-NLS-1$

    /**
     * The java system property for the https proxy password.
     * 
     * @since 0.8.1
     */
    public static final String BUILD_CLIENT_HTTPS_PROXY_PASSWORD_PROPERTY = "https.proxyPassword"; //$NON-NLS-1$
    
    /**
     * The java system property for the http non proxy hosts
     * 
     * @since 0.11.0.2
     *  (6.0.2)
     */
    public static final String BUILD_CLIENT_HTTP_NO_PROXY_HOST_PROPERTY = "http.nonProxyHosts"; //$NON-NLS-1$
    
    /**
     * The java system property for the https non proxy hosts
     * 
     * @since 0.11.0.2
     *  (6.0.2)
     */
    public static final String BUILD_CLIENT_HTTPS_NO_PROXY_HOST_PROPERTY = "https.nonProxyHosts"; //$NON-NLS-1$

    /**
     * Get the most recent build result in one of the specified states for the
     * specified build definition based on the build start time.
     * 
     * @param buildDefinitionHandle
     *            The build definition to find the most recent build result in
     *            one of the specified states for.
     * @param buildStates
     *            The build states of interest.
     * @param properties
     *            The properties of the build result to retrieve.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return <code>null</code> if the given
     *         <code>buildDefinitionHandle</code> does not exist or if there
     *         are no builds of that type in one of the specified states.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinitionHandle</code> is
     *             <code>null</code> or <code>properties</code> contains
     *             <code>null</code> slots.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildResult getLastBuildResult(IBuildDefinitionHandle buildDefinitionHandle, BuildState[] buildStates,
            String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Get the build engine with the given id. If more than one build engine has
     * the same id, only the first engine found is returned. If no build engine
     * has the id, return null.
     * 
     * @param id
     *            the build engine id
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return a build engine
     * @throws IllegalArgumentException
     *             If the given <code>id</code> is empty or <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildEngine getBuildEngine(String id, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Retrieves the build definition with the given id.
     * 
     * @param buildDefinitionId
     *            The build definition to retrieve.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build definition, or <code>null</code> if no build
     *         definition with the given id is found.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinitionId</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildDefinition getBuildDefinition(String buildDefinitionId, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Retrieve the last time a build engine contacted the repository.
     * 
     * @param buildEngineHandle
     *            The engine whose last contact time will be checked. The engine
     *            must exist in the repository.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The last time the engine contacted the repository. Return null if
     *         the service has never contacted the repository.
     * @throws IllegalArgumentException
     *             If the <code>buildEngineHandle</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public Timestamp getLastContactTime(IBuildEngineHandle buildEngineHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Retrieve the time since an engine contacted the repository. The result is
     * approximate. Time to contact the server is not taken into account.
     * 
     * @param buildEngineHandle
     *            The engine whose last contact time will be checked. The engine
     *            must exist in the repository.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The number of milliseconds since the last time the engine
     *         contacted the repository. Return -1 if the engine has never
     *         contacted the repository.
     * @throws IllegalArgumentException
     *             If the <code>buildEngineHandle</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @since 0.8
     */
    public long getTimeSinceLastContact(IBuildEngineHandle buildEngineHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Set the last time the engine contacted the repository.
     * 
     * @param buildEngineHandle
     *            The engine whose last contact time will be updated.
     * @param lastContactTime
     *            The last time the engine contacted the repository. This can be
     *            <code>null</code> to indicate the engine should be updated
     *            to the current server time where the engine is stored.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return the build engine whose last contact time was updated
     * @throws IllegalArgumentException
     *             If <code>buildEngineHandle</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildEngine setLastContactTime(IBuildEngineHandle buildEngineHandle, Timestamp lastContactTime,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Retrieves the build result contributions matching the specified extended
     * contribution type id.
     * 
     * @param buildResultHandle
     *            The build result to retrieve the build result contributions
     *            from.
     * @param extendedContributionTypeId
     *            The id of the extended contribution type to retrieve.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result contributions matching the specified extended
     *         contribution type id
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or
     *             <code>extendedContributionTypeId</code> is
     *             <code>null</code>.
     */
    public IBuildResultContribution[] getBuildResultContributions(IBuildResultHandle buildResultHandle,
            String extendedContributionTypeId, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Retrieves the build result contributions matching the specified extended
     * contribution type ids.
     * 
     * @param buildResultHandle
     *            The build result to retrieve the build result contributions
     *            from.
     * @param extendedContributionTypeIds
     *            The ids of the extended contribution types to retrieve.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result contributions matching the specified extended
     *         contribution type ids
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or any member of the
     *             <code>extendedContributionTypeIds</code> array is
     *             <code>null</code>.
     */
    public IBuildResultContribution[] getBuildResultContributions(IBuildResultHandle buildResultHandle,
            String[] extendedContributionTypeIds, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Adds the specified build result contributions to the given build result.
     * The build result itself will only be modified under two conditions:
     * <ol>
     * <li>The build result does not yet reference a container that is used to
     * store the contributions.</li>
     * <li>One of the contributions indicates that it impacts the primary
     * result and has a {@link BuildStatus} of greater severity than the status
     * of the specified build result.</li>
     * </ol>
     * 
     * @param buildResultHandle
     *            The build result to add the contributions to.
     * @param buildResultContributions
     *            The build result contributions to add to the build result.
     * @param properties
     *            The properties to return in the build result if it's modified.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result with the specified properties that the
     *         contributions were added to.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or any member of the
     *             <code>buildResultContributions</code> or
     *             <code>properties</code> array is <code>null</code>.
     */
    public IBuildResult addBuildResultContributions(IBuildResultHandle buildResultHandle,
            IBuildResultContribution[] buildResultContributions, String[] properties, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Adds the specified build result contribution to the given build result.
     * The build result itself will only be modified under two conditions:
     * <ol>
     * <li>The build result does not yet reference a container that is used to
     * store the contribution.</li>
     * <li>The contribution indicates that it impacts the primary result and
     * has a {@link BuildStatus} of greater severity than the status of the
     * specified build result.</li>
     * </ol>
     * 
     * @param buildResultHandle
     *            The build result to add the contribution to.
     * @param buildResultContribution
     *            The build result contribution to add to the build result.
     * @param properties
     *            The properties to return in the build result if it's modified.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result with the specified properties that the
     *         contribution was added to.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or any member of the
     *             <code>buildResultContributions</code> or
     *             <code>properties</code> array is <code>null</code>.
     */
    public IBuildResult addBuildResultContribution(IBuildResultHandle buildResultHandle,
            IBuildResultContribution buildResultContribution, String[] properties, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Adds the specified build result contributions to the given build result.
     * The build result itself will only be modified under two conditions:
     * <ol>
     * <li>The build result does not yet reference a container that is used to
     * store the contributions.</li>
     * <li>One of the contributions indicates that it impacts the primary
     * result and has a {@link BuildStatus} of greater severity than the status
     * of the specified build result.</li>
     * </ol>
     * 
     * @param buildResultHandle
     *            The build result to add the contributions to.
     * @param buildResultContributions
     *            The build result contributions to add to the build result.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or any member of the
     *             <code>buildResultContributions</code> array is
     *             <code>null</code>.
     */
    public void addBuildResultContributions(IBuildResultHandle buildResultHandle,
            IBuildResultContribution[] buildResultContributions, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Adds the specified build result contribution to the given build result.
     * The build result itself will only be modified under two conditions:
     * <ol>
     * <li>The build result does not yet reference a container that is used to
     * store the contribution.</li>
     * <li>The contribution indicates that it impacts the primary result and
     * has a {@link BuildStatus} of greater severity than the status of the
     * specified build result.</li>
     * </ol>
     * 
     * @param buildResultHandle
     *            The build result to add the contribution to.
     * @param buildResultContribution
     *            The build result contribution to add to the build result.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or any member of the
     *             <code>buildResultContributions</code> array is
     *             <code>null</code>.
     */
    public void addBuildResultContribution(IBuildResultHandle buildResultHandle,
            IBuildResultContribution buildResultContribution, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Deletes the specified build result contributions from the given build
     * result. The contributions must be instances that were returned by a call
     * to
     * {@link #getBuildResultContributions(IBuildResultHandle, String, IProgressMonitor)}
     * or
     * {@link #getBuildResultContributions(IBuildResultHandle, String[], IProgressMonitor)}.
     * The deletion of contributions will have no automatic impact on the status
     * of the build result. It's up to the client to determine if the deletion
     * warrants a change in the status of the build result.
     * 
     * @param buildResultHandle
     *            The build result to remove the contributions from.
     * @param buildResultContributions
     *            The build result contributions to remove from the build
     *            result.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or any member of the
     *             <code>buildResultContributions</code> array is
     *             <code>null</code>.
     */
    public void deleteBuildResultContributions(IBuildResultHandle buildResultHandle,
            IBuildResultContribution[] buildResultContributions, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Start a build activity with the label in the specified build result.
     * Returns a unique id for the newly started activity.
     * <p>
     * If <code>parentId</code> is set, the new activity is created as a child
     * of the existing activity with that id.
     * <p>
     * If <code>autoComplete</code> is true, then this activity will be
     * automatically completed when either the build is completed, this
     * activity's parent activity is completed or when the next peer activity is
     * started.
     * <p>
     * 
     * @param buildResultHandle
     *            The build result to start the activity in.
     * @param label
     *            The label describing the activity.
     * @param parentId
     *            Optional id of a parent activity.
     * @param autoComplete
     *            Whether or not to automatically complete this activity.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The unique id of the new activity.
     * @throws TeamBuildStateException
     *             If the build is not currently in progress.
     * @throws TeamBuildException
     *             If <code>parentId</code> is set but an activity with that
     *             id is not found or is found but has already completed.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or <code>label</code>
     *             is <code>null</code>, or if <code>label</code> is empty.
     */
    public String startBuildActivity(IBuildResultHandle buildResultHandle, String label, String parentId,
            boolean autoComplete, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Complete the build activity with the specified id in the specified build
     * result.
     * 
     * @param buildResultHandle
     *            The build result to complete the activity in.
     * @param id
     *            The id of the activity to complete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> or <code>id</code> is
     *             <code>null</code> or if <code>id</code> is empty.
     */
    public void completeBuildActivity(IBuildResultHandle buildResultHandle, String id, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Retrieves the build activities from the specified build result. Returns
     * an array containing the build's root activities. Nested activities can be
     * accessed by calling {@link IBuildActivity#getChildActivities()} on the
     * activities in the list.
     * 
     * @param buildResultHandle
     *            The build result to retrieve the build activities from.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of the root build activities contained in the specified
     *         build result. May return an empty array but will never return
     *         null.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> is <code>null</code>.
     */
    public IBuildActivity[] getBuildActivities(IBuildResultHandle buildResultHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Retrieves the in progress activity label for the specified build result.
     * The text will be one of the following:
     * <ol>
     * <li>No activity in progress - an empty string will be returned.</li>
     * <li>Single activity in progress - The label of that activity will be
     * returned.</li>
     * <li>Multiple activities in progress - the text "multiple activities"
     * will be returned.</li>
     * </ol>
     * 
     * @param buildResultHandle
     *            The build result to retrieve the build activities from.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return Text representing the in progress activities.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> is <code>null</code>.
     * @deprecated Use
     *             {@link #getBuildActivities(IBuildResultHandle, IProgressMonitor) }
     *             to traverse the activities to present an appropriate in
     *             progress activity label. It will be removed in a future
     *             iteration.
     */
    public String getInProgressActivityLabel(IBuildResultHandle buildResultHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Retrieves a number representing the completeness of the specified build
     * result as a percentage. The number returned will be from 0 to 100
     * inclusive, or -1 to indicate there is no progress available.
     * 
     * @param buildResultHandle
     *            The build result to retrieve the build activities from.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A number from 0 to 100 representing the completeness of the build
     *         as a percentage, or -1 to indicate there is no progress
     *         available.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildResultHandle</code> is <code>null</code>.
     * @deprecated Use a combination of
     *             {@link #getBuildActivities(IBuildResultHandle, IProgressMonitor)}
     *             and {@link IBuildDefinition#getBuildAverageData()} to compute
     *             the percent complete. This method will be removed in a future
     *             iteration. It's use is <b>not</b> encouraged.
     */
    public int getPercentComplete(IBuildResultHandle buildResultHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Gets all tags that are on builds in the given project areas. (A build is
     * in the project area if its build definition is associated with a team
     * area in the project area).
     * 
     * @param projectAreaHandles
     *            Array of {@link IProjectAreaHandle} that scope the tags.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return Array of {@link String}; may be empty if there are no tagged
     *         builds associated with the given <code>projectAreaHandles.</code>
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             if projectAreaHandles, is null OR contains null elements.
     */
    public String[] getTags(IProjectAreaHandle[] projectAreaHandles, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Gets all tags that are on builds for the given build definitions; if
     * <code>definitionHandles</code> is <code>null</code> or empty, gets
     * all tags in the repository.
     * 
     * @param definitionHandles
     *            Array of {@link IBuildDefinitionHandle} that scope the tags;
     *            may be <code>null</code> but may not contain null elements.
     *            If <code>definitionHandles</code> is null or empty, this
     *            method will retrieve all tags in the repository.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return Array of {@link String}; may be empty if there are no tagged
     *         builds associated with the given <code>definitionHandles</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the <code>definitionHandles</code> array contains any
     *             null slots.
     */
    public String[] getTags(IBuildDefinitionHandle[] definitionHandles, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Determines if there are any responsive active engines that support the
     * specified build definition. An engine is considered responsive if any of
     * the following conditions are met:
     * <p>
     * <ul>
     * <li>The engine is not participating in last contact threshold
     * monitoring.</li>
     * <li>The engine is currently busy doing another build.</li>
     * <li>The difference between the last time the engine contacted the
     * repository and the time this method is invoked does not exceed the
     * monitoring threshold.</li>.
     * 
     * @param buildDefinitionHandle
     *            The build definitions to determine if there are any responsive
     *            engines for.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return <code>true</code> if at least one engine supporting the
     *         specified definition is considered responsive.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinitionHandle</code> is
     *             <code>null</code>.
     */
    public boolean anyResponsiveBuildEngines(IBuildDefinitionHandle buildDefinitionHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Determines if there are any active engines that support the specified
     * build definition and have request processing enabled.
     * <p>
     * If a definition has no supporting build engines with request processing
     * enabled, then builds for that definition should not be scheduled or
     * manually requested because the requests will never be processed.
     * </p>
     * 
     * @param buildDefinitionHandle
     *            The build definition to determine if there are any
     *            request-processing-enabled engines for.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return <code>true</code> if at least one engine supporting the
     *         specified definition supports request processing
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinitionHandle</code> is
     *             <code>null</code>.
     */
    public boolean anyBuildEnginesWithRequestProcessingEnabled(IBuildDefinitionHandle buildDefinitionHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;
    
    /**
     * Moves the given items to the specified folder.
     * 
     * @param itemHandles
     *            The array of items to move.
     * @param folderHandle
     *            The folder to move to. If <code>null</code>, then the items
     *            will be uncategorized.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>itemHandles</code> is <code>null</code> or contains
     *             <code>null</code> elements.
     * @since 0.9.6
     */
    public void moveItemsToFolder(IItemHandle[] itemHandles, IBuildFolderHandle folderHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;
    
    /**
     * Gets children of the specified folder that have a process area within
     * <code>processAreaHandles</code>. This includes both build folders and
     * build definitions.
     * 
     * @param buildFolderHandle
     *            The folder to query children of. If <code>null</code>, then
     *            children without a parent are returned (i.e. uncategorized
     *            build definitions and folders).
     * @param processAreaHandles
     *            The process areas to search for children. Must not be
     *            <code>null</code> or contain <code>null </code> elements.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of {@link IBuildItemNamePair}s that each contain the
     *         <code>name</code> and <code>itemHandle</code> corresponding to a
     *         child of the specified folder.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>processAreaHandles</code> is <code>null</code> or
     *             contains <code>null</code> elements.
     * @since 0.9.6
     */
    public IBuildItemNamePair[] getChildrenOfFolder(IBuildFolderHandle buildFolderHandle,
            IProcessAreaHandle[] processAreaHandles, IProgressMonitor progressMonitor) throws TeamRepositoryException;
    
    /**
     * Returns an array of item-name pairs leading from the containing folder of
     * the specified <code>itemHandle</code> to the root folder. An empty array
     * indicates that the item is uncategorized.
     * 
     * @param itemHandle
     *            The item to retrieve the folders of.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A possibly empty array of item-name pairs of type
     *         {@link IBuildItemNamePair} that each contain the
     *         <code>name</code> and <code>itemHandle</code> of the ancestor.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws PermissionDeniedException
     *             If read-access to one of the ancestor build folders is
     *             denied.
     * @throws IllegalArgumentException
     *             If the <code>itemHandle</code> is <code>null</code>.
     * @since 0.9.6
     */
    public IBuildItemNamePair[] getAncestorFolders(IItemHandle itemHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

}
