/*******************************************************************************
 *  Licensed Materials - Property of IBM
 *  (c) Copyright IBM Corporation 2006, 2014. All Rights Reserved.
 *  
 *  Note to U.S. Government Users Restricted Rights:
 *  Use, duplication or disclosure restricted by GSA ADP Schedule
 *  Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.client;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.TeamBuildStateException;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildRequestHandle;
import com.ibm.team.build.common.model.IBuildRequestParams;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * The <code>ITeamBuildRequestClient</code> interface provides ability to
 * create, retrieve, and manipulate build requests.
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface ITeamBuildRequestClient extends ITeamBuildBaseClient {

    /**
     * Provides the ability to request that a build should occur for a
     * particular build definition. Requests will be granted only for build
     * definitions that have at least one active build engine to support them.
     * <p>
     * Calling this method results in a build that uses the default build
     * properties specified in the build definition.
     * 
     * @param buildDefinitionHandle
     *            The build definition to request a build for.
     * @param allowDuplicateRequests
     *            Indicates if a duplicate request can be made if one for the
     *            type already exists in the queue.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return
     *            <ul>
     *            <li>The new build request for the specified build definition.</li>
     *            <li>The existing build request if one already exists for the
     *            build definition and <code>allowDuplicateRequests</code> is
     *            <code>false</code>.</li>
     *            <li><code>null</code> if the request could not be honored.</li>
     *            </ul>
     * @throws TeamRepositoryException
     *             If error occurs when requesting the build.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> or any of the slots in
     *             <code>buildProperties</code> are <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest requestBuild(IBuildDefinitionHandle buildDefinitionHandle, boolean allowDuplicateRequests,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Provides the ability to request that a build should occur for a
     * particular build definition. Requests will be granted only for build
     * definitions that have at least one active build engine to support them.
     * <p>
     * This method allows the user to specify a list of additional or modified
     * build properties that will override properties included in the build
     * definition.
     * 
     * @param buildDefinitionHandle
     *            The build definition to request a build for.
     * @param buildProperties
     *            Any new or overridden build properties that should be added or
     *            replaced as part of the request. This parameter can be
     *            <code>null</code> if the default build properties associated
     *            with the build definition should be used.
     * @param allowDuplicateRequests
     *            Indicates if a duplicate request can be made if one for the
     *            type already exists in the queue.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return
     *            <ul>
     *            <li>The new build request for the specified build definition.</li>
     *            <li>The existing build request if one already exists for the
     *            build definition and <code>allowDuplicateRequests</code> is
     *            <code>true</code>.</li>
     *            <li><code>null</code> if the request could not be honored.</li>
     *            </ul>
     * @throws TeamRepositoryException
     *             If error occurs when requesting the build.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> or any of the slots in
     *             <code>buildProperties</code> are <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest requestBuild(IBuildDefinitionHandle buildDefinitionHandle, IBuildProperty[] buildProperties,
            boolean allowDuplicateRequests, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Provides the ability to request that a build should occur for a
     * particular build definition. Requests will be granted only for build
     * definitions that have at least one active build engine to support them.
     * <p>
     * This method allows the user to specify build properties that should be
     * deleted from the requested build as well as new or modified properties.
     * Should a property be present in both lists, the deletion list will take
     * priority and the build property will be deleted.
     * 
     * @param buildDefinitionHandle
     *            The build definition to request a build for.
     * @param newOrModifiedBuildProperties
     *            Any new or overridden build properties that should be added or
     *            replaced as part of the request. This parameter may be
     *            <code>null</code> if no build properties should be added or
     *            modified.
     * @param deletedBuildProperties
     *            Any build propertied that should be deleted as part of the
     *            request. This parameter may be null if no build properties
     *            should be deleted.
     * @param allowDuplicateRequests
     *            Indicates if a duplicate request can be made if one for the
     *            type already exists in the queue.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return
     *            <ul>
     *            <li>The new build request for the specified build definition.</li>
     *            <li>The existing build request if one already exists for the
     *            build definition and <code>allowDuplicateRequests</code> is
     *            <code>true</code>.</li>
     *            <li><code>null</code> if the request could not be honored.</li>
     *            </ul>
     * @throws TeamRepositoryException
     *             If error occurs when requesting the build.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> or any of the slots in
     *             <code>buildProperties</code> are <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest requestBuild(IBuildDefinitionHandle buildDefinitionHandle,
            IBuildProperty[] newOrModifiedBuildProperties, IBuildProperty[] deletedBuildProperties,
            boolean allowDuplicateRequests, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Provides the ability to request that a build should occur for a
     * particular build definition. Requests will be granted only for build
     * definitions that have at least one active build engine to support them.
     * <p>
     * This method allows the user to specify build properties that should be
     * deleted from the requested build as well as new or modified properties.
     * Should a property be present in both lists, the deletion list will take
     * priority and the build property will be deleted.
     * 
     * @param buildDefinitionHandle
     *            The build definition to request a build for.
     * @param newOrModifiedBuildProperties
     *            Any new or overridden build properties that should be added or
     *            replaced as part of the request. This parameter may be
     *            <code>null</code> if no build properties should be added or
     *            modified.
     * @param deletedBuildProperties
     *            Any build propertied that should be deleted as part of the
     *            request. This parameter may be null if no build properties
     *            should be deleted.
     * @param allowDuplicateRequests
     *            Indicates if a duplicate request can be made if one for the
     *            type already exists in the queue.
     * @param personalBuild
     *            Indicates if this is a request for a personal build.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return
     *            <ul>
     *            <li>The new build request for the specified build definition.</li>
     *            <li>The existing build request if one already exists for the
     *            build definition and <code>allowDuplicateRequests</code> is
     *            <code>true</code>.</li>
     *            <li><code>null</code> if the request could not be honored.</li>
     *            </ul>
     * @throws TeamRepositoryException
     *             If error occurs when requesting the build.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> or any of the slots in
     *             <code>buildProperties</code> are <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest requestBuild(IBuildDefinitionHandle buildDefinitionHandle,
            IBuildProperty[] newOrModifiedBuildProperties, IBuildProperty[] deletedBuildProperties,
            boolean allowDuplicateRequests, boolean personalBuild, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Provides the ability to request that a build should occur for a
     * particular build definition. Requests will be granted only for build
     * definitions that have at least one active build engine to support them,
     * except when engines are explicitly listed as potential handlers.
     * 
     * @param params
     *            The build request parameters.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return <ul>
     *         <li>The new build request for the specified build definition.</li>
     *         <li>The existing build request if one already exists for the
     *         build definition and <code>allowDuplicateRequests</code> is
     *         <code>true</code>.</li>
     *         <li><code>null</code> if the request could not be honored.</li>
     *         </ul>
     * @throws TeamRepositoryException
     *             If error occurs when requesting the build.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> or any of the slots in
     *             <code>buildProperties</code> are <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     * @since 0.8
     */
    public IBuildRequest requestBuild(IBuildRequestParams params, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Requests a build by creating a build request for the specified build
     * definition and automatically setting the request into a processed and in
     * progress state specifying the supplied engine as the handling process. No
     * other potential handlers will be given a chance to process the request.
     * 
     * @param buildDefinitionHandle
     *            The build definition to request a build to start for.
     * @param buildEngineHandle
     *            The engine used to process the build.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build request that was used to start the build.
     * @throws TeamRepositoryException
     *             If an error occurs when retrieving the pending requests.
     * @throws TeamBuildException
     *             If a repository error occurs requesting and processing the
     *             build.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinitionHandle</code> or
     *             <code>buildEngineHandle</code> is <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest requestAndStartBuild(IBuildDefinitionHandle buildDefinitionHandle,
            IBuildEngineHandle buildEngineHandle, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            TeamBuildException, IllegalArgumentException;

    /**
     * Cancels a build request that is pending.
     * 
     * @param buildRequestHandle
     *            The request to cancel.
     * @param properties
     *            The properties to fetch in the build request that is canceled.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The request that was cancelled or <code>null</code> if it
     *         doesn't exist or is not pending.
     * @throws TeamRepositoryException
     *             If error occurs when cancelling the request.
     * @throws TeamBuildStateException
     *             If an attempt is made cancel a pending build that is in any
     *             state other than "Not Started".
     * @throws IllegalArgumentException
     *             If the <code>IBuildRequestHandle</code> is
     *             <code>null</code> or any of the slots in
     *             <code>properties</code> are <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest cancelPendingRequest(IBuildRequestHandle buildRequestHandle, String[] properties,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, TeamBuildStateException,
            IllegalArgumentException;

    /**
     * Marks the specified build result an incomplete build.
     * 
     * @param buildResultHandle
     *            The build result to make incomplete.
     * @param properties
     *            The properties to fetch in the build result that is made
     *            incomplete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result made incomplete.
     * @throws TeamRepositoryException
     *             If an exception occurs marking the build incomplete.
     * @throws TeamBuildStateException
     *             If an attempt is made to make a build incomplete that is in
     *             any state other than "In Progress".
     * @throws IllegalArgumentException
     *             If the <code>buildResultHandle</code> is <code>null</code>
     *             or any of the slots in <code>properties</code> are
     *             <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildResult makeBuildIncomplete(IBuildResultHandle buildResultHandle, String[] properties,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, TeamBuildStateException,
            IllegalArgumentException;

    /**
     * Marks the specified build result as a completed build unless
     * <code>buildNotNessary</code> is <code>true</code>. The
     * <code>buildNotNessary</code> parameter provides the ability to allow a
     * build to be requested but then be completed as not necessary if it is
     * later determine that the build did not need to occur(for example, source
     * control was checked and no changes were found so the build doesn't
     * occur).
     * 
     * @param buildResultHandle
     *            The build result to make complete.
     * @param buildNotNecessary
     *            Indicates that a build that was requested was determined not
     *            to be necessary.
     * @param properties
     *            The properties to fetch in the build result that is made
     *            complete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result made complete or <code>null</code> if the
     *         build was made not necessary.
     * @throws TeamRepositoryException
     *             If an exception occurs making the build complete.
     * @throws TeamBuildStateException
     *             If an attempt is made to complete a build as not necessary
     *             when it's in a state other than "Not Started" or complete a
     *             necessary build in any state other than "In Progress".
     * @throws IllegalArgumentException
     *             If the <code>buildResultHandle</code> is <code>null</code>
     *             or any of the slots in <code>properties</code> are
     *             <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildResult makeBuildComplete(IBuildResultHandle buildResultHandle, boolean buildNotNecessary,
            String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            TeamBuildStateException, IllegalArgumentException;

    /**
     * Mark the build as complete with the status provided as long as the status is more severe
     * than the build result's current status. If the status is a 
     * warning and the build was set to ignore warnings, then the
     * status will not be updated.
     * 
     * @param buildResultHandle
     *            The build result to make complete
     * @param buildStatus 
     *            The status set on the build result providing it is more severe than the current
     *            status of the build. If its the warning status, for it to have
     *            an effect the build must be configured to not ignore warnings.
     * @param properties
     *            The properties to fetch in the build result that is made
     *            complete.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result made complete.
     * @throws TeamRepositoryException
     *             If an exception occurs making the build complete.
     * @throws TeamBuildStateException
     *             If an attempt is made to complete a build as not necessary
     *             when it's in a state other than "Not Started" or complete a
     *             necessary build in any state other than "In Progress".
     * @throws IllegalArgumentException
     *             If the <code>buildResultHandle</code> is <code>null</code>
     *             or any of the slots in <code>properties</code> are
     *             <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     * @since 0.10.0.1 (RTC 5.0.1)
     */
    public IBuildResult makeBuildCompleteWithStatus(IBuildResultHandle buildResultHandle, BuildStatus buildStatus, String[] properties,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, TeamBuildStateException, IllegalArgumentException;

    /**
     * Retrieves the next request in the queue for the specified build engine If
     * one exists it will be placed into a state indicating it as been removed
     * from the queue for processing.
     * 
     * @param buildEngineHandle
     *            The build engine handle to retrieve requests for.
     * @param properties
     *            The properties to fetch in the build request.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The next request in the queue for the specified build engine or
     *         <code>null</code> if one doesn't exist.
     * @throws TeamRepositoryException
     *             If an error occurs when retrieving the queued request.
     * @throws IllegalArgumentException
     *             If the <code>buildEngineHandle</code> is <code>null</code>
     *             or any of the slots in <code>properties</code> are
     *             <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest getNextRequest(IBuildEngineHandle buildEngineHandle, String[] properties,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Claims the specified request from the queue. It will be placed into a
     * state indicating it has been removed from the queue for processing.
     * 
     * @param buildRequestHandle
     *            The build request handle to retrieve.
     * @param buildEngineHandle
     *            The build engine handle to retrieve requests for.
     * @param properties
     *            The properties to fetch in the build request.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The processed request for the specified build request handle or
     *         <code>null</code> if the request was already claimed from the queue.
     * @throws TeamRepositoryException
     *             If an error occurs when retrieving the request.
     * @throws IllegalArgumentException
     *             If the <code>buildEngineHandle</code> is <code>null</code> or
     *             any of the slots in <code>properties</code> are
     *             <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     * @since 0.8
     */
    public IBuildRequest claimRequest(IBuildRequestHandle buildRequestHandle, IBuildEngineHandle buildEngineHandle,
            String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;
    
    /**
     * Retrieves the next request in the queue for the specified build engine
     * and definition. If one exists it will be placed into a state indicating
     * it as been removed from the queue for processing.
     * 
     * @param buildDefinitionId
     *            The id of the build definition to retrieve requests for.
     * @param buildEngineHandle
     *            The build engine handle to retrieve requests for.
     * @param properties
     *            The properties to fetch in the build request.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The next request in the queue for the specified build engine or
     *         <code>null</code> if one doesn't exist.
     * @throws TeamRepositoryException
     *             If an error occurs when retrieving the queued request.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinitionId</code> or
     *             <code>buildEngineHandle</code> is <code>null</code> or
     *             any of the slots in <code>properties</code> are
     *             <code>null</code>.
     * @throws IllegalArgumentException
     *             If the <code>buildEngine</code> is <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildRequest getNextRequest(String buildDefinitionId, IBuildEngineHandle buildEngineHandle,
            String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Starts the build for the specified build request.
     * 
     * @param buildRequestHandle
     *            The build request to start the build for.
     * @param properties
     *            The properties to fetch in the build result that is associated
     *            with the build that is started.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result associated with the started build or
     *         <code>null</code> if it was already started.
     * @throws TeamRepositoryException
     *             If an error occurs when starting the build.
     * @throws TeamBuildStateException
     *             If an attempt is made to start a build using a request that
     *             was not returned by either
     *             {@link #getNextRequest(IBuildEngineHandle, String[], IProgressMonitor)}
     *             or
     *             {@link #getNextRequest(String, IBuildEngineHandle, String[], IProgressMonitor)}
     *             or a request that has already been started.
     * @throws IllegalArgumentException
     *             If the <code>buildRequestHandle</code> or any of the slots
     *             in <code>properties</code> are <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildResult startBuild(IBuildRequestHandle buildRequestHandle, String[] properties,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, TeamBuildStateException,
            IllegalArgumentException;

}