/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.client.iterator;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.internal.client.ITeamBuildRecordClient;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Provides the ability to iterate through the build result records associated
 * with a set of build results.
 */
public interface IBuildResultRecordIterator {

    /**
     * The maximum number of build result records that can be retrieved
     * in one invocation of {@link #next(int, IProgressMonitor)}.
     */
    public static int MAX_BUILD_RESULT_RECORDS = ITeamBuildRecordClient.MAX_BUILD_RESULT_RECORDS;

    /**
     * Determines if the iterator has additional build result records remaining
     * to iterate through.
     * 
     * @return <code>true</code> if there are additional build result records
     *         remaining to iterate through.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public boolean hasNext() throws TeamRepositoryException;

    /**
     * Returns the current build result record and positions the iterator at the
     * next record.
     * 
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The current build result record the iterator points at.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildResultRecord next(IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Returns the next <code>numRecords</code> of build result records and
     * positions the iterator at the next record beyond them. Less than the
     * desired number of records may be returned if there are not enough records
     * left to fulfill the request.
     * 
     * @param numRecords
     *            The number of build result records to retrieve.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The next <code>numRecords</code> of build result records
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the specified <code>numRecords</code> exceed
     *             {@link #MAX_BUILD_RESULT_RECORDS}.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     */
    public IBuildResultRecord[] next(int numRecords, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Indicates the total number of build result records that iterator is
     * capable of traversing through.
     * 
     * @return the total number of build result records that iterator is capable
     *         of traversing through.
     */
    public int size();

}
