/*******************************************************************************
 *  Licensed Materials - Property of IBM
 *  (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved.
 *  
 *  Note to U.S. Government Users Restricted Rights:
 *  Use, duplication or disclosure restricted by GSA ADP Schedule
 *  Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.client.ITeamBuildBaseClient;
import com.ibm.team.build.common.TeamBuildStateException;
import com.ibm.team.build.common.model.IBuildCategoryHandle;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.common.model.IBuildItem;
import com.ibm.team.build.common.model.IBuildItemHandle;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.common.model.IBuildFolder;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.common.model.ICompileContributionHandle;
import com.ibm.team.build.common.model.ICompilePackage;
import com.ibm.team.build.common.model.ICompilePackageHandle;
import com.ibm.team.build.common.model.IJUnitTestClass;
import com.ibm.team.build.common.model.IJUnitTestClassHandle;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.common.model.IJUnitTestSuiteContributionHandle;
import com.ibm.team.build.internal.common.ITeamBuildService;
import com.ibm.team.build.internal.common.helper.PropertyHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.process.client.IProcessClientService;
import com.ibm.team.process.common.advice.IItemsResponse;
import com.ibm.team.process.common.advice.IOperationReport;
import com.ibm.team.process.common.advice.ProcessRunnable;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.util.EventSource;
import com.ibm.team.repository.client.util.IClientLibraryContext;
import com.ibm.team.repository.client.util.IClientLibraryContext.IServiceRunnable;
import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.repository.common.query.IDataQuery;
import com.ibm.team.repository.common.query.IDataQueryPage;
import com.ibm.team.repository.common.query.IItemQuery;
import com.ibm.team.repository.common.query.IItemQueryPage;
import com.ibm.team.repository.common.query.IQueryPage;
import com.ibm.team.repository.common.service.IQueryService;

/**
 * The abstract client library implementation for all team build services.
 * <p>
 * <p>
 * The implementation uses an event source to protect its operations that
 * interact with the item manager. The assumption is that the derived build
 * client libraries are the primary source of shared items for all build item
 * types. By acquiring the event source lock, the build client libraries ensures
 * that they will not be a source of concurrent updates to the shared items.
 */
public abstract class AbstractTeamBuildClient extends EventSource implements ITeamBuildBaseClient {

    private IClientLibraryContext fContext;

    /**
     * Creates the abstract team build client for a specified context.
     * 
     * @param context
     *            The context to create the abstract team build client for.
     */
    public AbstractTeamBuildClient(IClientLibraryContext context) {
        fContext = context;

    }

    /**
     * Retrieves the context used to create the client libraries.
     * 
     * @return The context used to create the client libraries.
     */
    protected IClientLibraryContext getContext() {
        return fContext;
    }

    protected ITeamRepository getRepository() {
        return fContext.teamRepository();
    }

    protected <T> T getService(Class<T> clazz) {
        return (T) getContext().getServiceInterface(clazz);
    }

    /**
     * Converts the specified array of properties to a <code>Collection</code>.
     * 
     * @param properties
     *            The array of properties to convert.
     * @return The specified array of properties to a <code>Collection</code>.
     */
    protected Collection toCollection(String[] properties) {
        if (properties == null) {
            properties = new String[0];
        }
        return Arrays.asList(properties);
    }

    /**
     * Add the (possible partial) item to the item manager cache.
     * 
     * @param item
     *            the item to add
     * @return the shared item that was added to the item manager cache
     * @throws TeamRepositoryException
     *             If something goes wrong when adding the item to the item
     *             manager.
     */
    protected IItem addToItemManager(IItem item) throws TeamRepositoryException {
        IItem sharedItem = null;
        if (null != item) {
            try {
                acquire();
                IItemManager itemManager = getRepository().itemManager();
                sharedItem = (IItem) itemManager.applyItemUpdates(Collections.singletonList(item)).get(0);
                if (sharedItem == null) {
                    /*
                     * The item manager rejected the update because the new
                     * partial item wasn't at least as full as the existing
                     * cached item. Refresh the cached item.
                     */
                    sharedItem = (IItem) itemManager.fetchPartialItem(item, IItemManager.REFRESH,
                            toCollection(PropertyHelper.getSetProperties(item)), null);
                }
            } finally {
                release();
            }
        }
        return sharedItem;
    }

    /**
     * Add the (possible partial) items to the item manager cache.
     * 
     * @param items
     *            the items to add
     * @return The shared items that were added to the item manager cache.
     *         Elements in the list are of the same concrete type as the given
     *         items.
     * @throws TeamRepositoryException
     *             If something goes wrong when adding the items to the item
     *             manager.
     */
    protected List addToItemManager(IItem[] items, IProgressMonitor progressMonitor) throws TeamRepositoryException {
        try {
            acquire();
            IItemManager itemManager = getRepository().itemManager();
            List sharedItems = itemManager.applyItemUpdatesOrRefresh(Arrays.asList(items), progressMonitor);
            return removeNullElements(sharedItems);
        } finally {
            release();
        }
    }

    private static final Collection fCollectionOfNull = Collections.singletonList(null);

    /**
     * Removes the <code>null</code> elements from the provided
     * <code>List</code>.
     * 
     * @param list
     *            a <code>List</code>; never <code>null</code>.
     * @return a <code>List</code>; may be empty but will not be<code>null</code>
     *         or contain <code>null</code> elements.
     */
    protected List removeNullElements(List list) {
        ValidationHelper.validateNotNull("list", list); //$NON-NLS-1$

        if (!list.contains(null)) {
            return list;
        } else {
            ArrayList prunedList = new ArrayList(list);
            prunedList.removeAll(fCollectionOfNull);
            return prunedList;
        }
    }

    /**
     * Helper method that informs this item manager that the given item has been
     * deleted from the repository. The item type must be either simple or
     * auditable. If the item manager has a shared item for an item, all
     * interested listeners are notified of the item's deletion. This method
     * does not create any new shared items. This method is intended to be used
     * by client library.
     * <p>
     * Usage note: This method does not directly delete a shared item from the
     * item manager. Rather, it tells listeners that the item has gone away.
     * Hopefully, they respond by dropping any references they have to the
     * shared item. If everyone does this, the shared item will be freed up and
     * eventually GC'd.
     * 
     * @param itemHandle
     *            the item handle to delete
     */
    protected void deleteFromItemManager(IItemHandle itemHandle) {
        try {
            acquire();
            getRepository().itemManager().applyItemDeletes(Collections.singletonList(itemHandle));
        } finally {
            release();
        }
    }

    /**
     * Helper method that informs this item manager that the given items have
     * been deleted from the repository. The item type must be either simple or
     * auditable. If the item manager has a shared item for an item, all
     * interested listeners are notified of the item's deletion. This method
     * does not create any new shared items. This method is intended to be used
     * by client library.
     * <p>
     * Usage note: This method does not directly delete a shared item from the
     * item manager. Rather, it tells listeners that the item has gone away.
     * Hopefully, they respond by dropping any references they have to the
     * shared item. If everyone does this, the shared item will be freed up and
     * eventually GC'd.
     * 
     * @param itemHandles
     *            a list of item handle (element type: IItemHandle); no
     *            <code>null</code> elements or duplicates; list itself must
     *            not be <code>null</code>
     */
    protected void deleteFromItemManager(IItemHandle[] itemHandles) {
        try {
            acquire();
            getRepository().itemManager().applyItemDeletes(Arrays.asList(itemHandles));
        } finally {
            release();
        }
    }

    /**
     * Helper method to invoke a service in with progress tracking and
     * the ability to cancel.
     * 
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @param serviceRunnable
     *            The service to invoke.
     * @return The results of the service call.
     * @throws TeamRepositoryException
     *             If a problem occurs interacting with the repository.
     */
    protected Object callCancelableService(IProgressMonitor progressMonitor, IServiceRunnable serviceRunnable)
            throws TeamRepositoryException {
        return getContext().callCancelableService(serviceRunnable, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildDefinition save(final IBuildDefinition buildDefinition, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        return (IBuildDefinition) internalSave(buildDefinition,
                Messages.AbstractTeamBuildClient_SAVE_BUILD_DEFINITION_OPERATION_NAME, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IJUnitTestSuiteContribution save(IJUnitTestSuiteContribution junitTestSuiteContribution,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        return (IJUnitTestSuiteContribution) internalSave(junitTestSuiteContribution,
                Messages.AbstractTeamBuildClient_SAVE_JUNIT_TEST_SUITE_OPERATION_NAME, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IJUnitTestClass save(IJUnitTestClass testClass, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return (IJUnitTestClass) internalSave(testClass,
                Messages.AbstractTeamBuildClient_SAVE_JUNIT_TEST_CLASS_OPERATION_NAME, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public ICompileContribution save(ICompileContribution compileContribution, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return (ICompileContribution) internalSave(compileContribution,
                Messages.AbstractTeamBuildClient_SAVE_COMPILE_CONTRIBUTION_OPERATION_NAME, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public ICompilePackage save(ICompilePackage compilePackage, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return (ICompilePackage) internalSave(compilePackage,
                Messages.AbstractTeamBuildClient_SAVE_COMPILE_PACKAGE_OPERATION_NAME, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildEngine save(IBuildEngine buildEngine, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return (IBuildEngine) internalSave(buildEngine,
                Messages.AbstractTeamBuildClient_SAVE_BUILD_ENGINE_OPERATION_NAME, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResult save(IBuildResult buildResult, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return (IBuildResult) internalSave(buildResult,
                Messages.AbstractTeamBuildClient_SAVE_BUILD_RESULT_OPERATION_NAME, progressMonitor);
    }

    @Override
    public IBuildFolder save(IBuildFolder buildFolder, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return (IBuildFolder) internalSave(buildFolder,
                Messages.AbstractTeamBuildClient_SAVE_BUILD_FOLDER_OPERATION_NAME, progressMonitor);
    }
    
    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(IBuildDefinitionHandle buildDefinitionHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        internalDelete(buildDefinitionHandle, Messages.AbstractTeamBuildClient_DELETE_BUILD_DEFINITION_OPERATION_NAME,
                progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(IJUnitTestSuiteContributionHandle junitTestSuiteContributionHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        internalDelete(junitTestSuiteContributionHandle,
                Messages.AbstractTeamBuildClient_DELETE_JUNIT_TEST_SUITE_OPERATION_NAME, progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(IJUnitTestClassHandle testClassHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        internalDelete(testClassHandle, Messages.AbstractTeamBuildClient_DELETE_JUNIT_TEST_CLASS_OPERATION_NAME,
                progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(ICompileContributionHandle compileContributionHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        internalDelete(compileContributionHandle,
                Messages.AbstractTeamBuildClient_DELETE_COMPILE_CONTRIBUTION_OPERATION_NAME, progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(ICompilePackageHandle compilePackageHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        internalDelete(compilePackageHandle, Messages.AbstractTeamBuildClient_DELETE_COMPILE_PACKAGE_OPERATION_NAME,
                progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(IBuildEngineHandle buildEngineHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        internalDelete(buildEngineHandle, Messages.AbstractTeamBuildClient_DELETE_BUILD_ENGINE_OPERATION_NAME,
                progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(IBuildResultHandle buildResultHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, TeamBuildStateException, IllegalArgumentException {
        internalDelete(buildResultHandle, Messages.AbstractTeamBuildClient_DELETE_BUILD_RESULT_OPERATION_NAME,
                progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void delete(IBuildCategoryHandle buildCategoryHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        internalDeleteWithoutProcess(buildCategoryHandle, progressMonitor);
    }
    
    @Override
    public void delete(final IBuildFolderHandle[] buildFolderHandles, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        class DeleteBuildItemRunnable extends ProcessRunnable {
            public IItemsResponse itemsResponse;
          
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                itemsResponse = getService(ITeamBuildService.class).deleteBuildFolders(buildFolderHandles);
                return null;
            }

        }

        callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                DeleteBuildItemRunnable runnable = new DeleteBuildItemRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.AbstractTeamBuildClient_DELETE_BUILD_FOLDER_OPERATION_NAME,
                        monitor);
                addToItemManager(runnable.itemsResponse.getClientItems(), progressMonitor);
                deleteFromItemManager(runnable.itemsResponse.getClientItems());
                deleteFromItemManager(buildFolderHandles);
                return null;
            }
        });

    }
    
    /**
     * Helper method to save a build item within the context of a server side
     * only process operation.
     * 
     * @param buildItem
     *            The build item to save.
     * @param operationId
     *            The operation id of the server side process operation.
     * @param progressMonitor
     *            The progress monitor to track progress on.
     * @return The latest state of the saved item.
     */
    private IBuildItem internalSave(final IBuildItem buildItem, final String operationId,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException {
        class SaveBuildItemRunnable extends ProcessRunnable {

            public IItem fSavedItem;

            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService(ITeamBuildService.class).save(buildItem);
                fSavedItem = response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildItem) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                SaveBuildItemRunnable runnable = new SaveBuildItemRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, operationId, progressMonitor);

                return addToItemManager(runnable.fSavedItem);
            }
        });

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IQueryPage fetchPage(final UUID token, final int startIndex, final int pageSize,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException {
        return (IQueryPage) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService(IQueryService.class).fetchPage(token, startIndex, pageSize);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IDataQueryPage queryData(final IDataQuery dataQuery, final Object[] parameters, final int pageSize,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException {
        return (IDataQueryPage) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService(IQueryService.class).queryData(dataQuery, parameters, pageSize);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IItemQueryPage queryItems(final IItemQuery itemQuery, final Object[] parameters, final int pageSize,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException {
        return (IItemQueryPage) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService(IQueryService.class).queryItems(itemQuery, parameters, pageSize);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IDataQueryPage uncommittedQueryData(final IDataQuery dataQuery, final Object[] parameters,
            final int pageSize, final IProgressMonitor progressMonitor) throws TeamRepositoryException {
        return (IDataQueryPage) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService(IQueryService.class).uncommittedQueryData(dataQuery, parameters, pageSize);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IItemQueryPage uncommittedQueryItems(final IItemQuery itemQuery, final Object[] parameters,
            final int pageSize, final IProgressMonitor progressMonitor) throws TeamRepositoryException {
        return (IItemQueryPage) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService(IQueryService.class).queryItems(itemQuery, parameters, pageSize);
            }
        });
    }

    /**
     * Helper method to delete a build item within the context of a server side
     * only process operation.
     * 
     * @param buildItemHandle
     *            The handle to the build item to delete.
     * @param operationId
     *            The operation id of the server side process operation.
     * @param progressMonitor
     *            The progress monitor to track progress on.
     * @throws TeamBuildStateException
     *             If an attempt is made to delete a build result that has been
     *             marked to not allow deletion.
     */
    private void internalDelete(final IBuildItemHandle buildItemHandle, final String operationId,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, TeamBuildStateException {
        class DeleteBuildItemRunnable extends ProcessRunnable {

            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService(ITeamBuildService.class).delete(buildItemHandle);
            }

        }

        callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                DeleteBuildItemRunnable runnable = new DeleteBuildItemRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, operationId, monitor);
                deleteFromItemManager((IItemHandle) buildItemHandle);
                return null;
            }
        });
    }

    /**
     * Helper method to delete a build item outside the context of a process
     * operation.
     * 
     * @param buildItemHandle
     *            The handle to the build item to delete.
     * @param progressMonitor
     *            The progress monitor to track progress on.
     */
    private void internalDeleteWithoutProcess(final IBuildItemHandle buildItemHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                getService(ITeamBuildService.class).delete(buildItemHandle);
                deleteFromItemManager((IItemHandle) buildItemHandle);
                return null;
            }
        });
    }

}