/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client;

import org.eclipse.core.runtime.ListenerList;

import com.ibm.team.build.internal.common.model.dto.BuildDefinitionStatusRecord;
import com.ibm.team.build.internal.common.model.dto.BuildResultRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildDefinitionStatusRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;

/**
 * A manager for {@link BuildResultRecord} and
 * {@link BuildDefinitionStatusRecord} events in the build client. Interested
 * parties can register to receive an event whenever a record is fetched from
 * the repository.
 * <p>
 * Listeners are notified on the same thread that fetched the record. Listeners
 * must not perform long operations.
 * </p>
 */
public class BuildRecordEventManager {
    private final ListenerList fBuildResultRecordListeners = new ListenerList();
    private final ListenerList fBuildDefinitionStatusRecordListeners = new ListenerList();

    private static BuildRecordEventManager fInstance = new BuildRecordEventManager();

    private BuildRecordEventManager() {
        // Don't allow construction.
    }

    /**
     * Get the singleton instance of the <code>BuildRecordManager</code>.
     * 
     * @return The singleton instance of the <code>BuildRecordManager</code>.
     */
    public static BuildRecordEventManager getInstance() {
        return fInstance;
    }

    /**
     * Add a listener that will be notified whenever an
     * <code>IBuildResultRecord</code> is fetched. The method has no effect if
     * the same listener is already registered.
     * 
     * @param listener
     *            The listener to add.
     */
    public void addListener(IBuildResultRecordListener listener) {
        fBuildResultRecordListeners.add(listener);
    }

    /**
     * Add a listener that will be notified whenever an
     * <code>IBuildDefinitionStatusRecord</code> is fetched. The method has no
     * effect if the same listener is already registered.
     * 
     * @param listener
     *            The listener to add.
     */
    public void addListener(IBuildDefinitionStatusRecordListener listener) {
        fBuildDefinitionStatusRecordListeners.add(listener);
    }

    /**
     * Remove a listener. The method has no effect if the same listener was not
     * already registered.
     * 
     * @param listener
     *            The listener to remove.
     */
    public void removeListener(IBuildResultRecordListener listener) {
        fBuildResultRecordListeners.remove(listener);
    }

    /**
     * Remove a listener. The method has no effect if the same listener was not
     * already registered.
     * 
     * @param listener
     *            The listener to remove.
     */
    public void removeListener(IBuildDefinitionStatusRecordListener listener) {
        fBuildDefinitionStatusRecordListeners.remove(listener);
    }

    /**
     * Notify the manager of records received. This method should only be called
     * by {@link TeamBuildRecordClient}.
     * 
     * @param records
     *            The records received.
     */
    public void notifyRecordsReceived(IBuildResultRecord[] records) {
        Object[] listeners = fBuildResultRecordListeners.getListeners();
        for (Object object : listeners) {
            IBuildResultRecordListener listener = (IBuildResultRecordListener) object;
            listener.recordsReceived(records);
        }
    }

    /**
     * Notify the manager of records received. This method should only be called
     * by {@link TeamBuildRecordClient}.
     * 
     * @param records
     *            The records received.
     */
    public void notifyRecordsReceived(IBuildDefinitionStatusRecord[] records) {
        Object[] listeners = fBuildDefinitionStatusRecordListeners.getListeners();
        for (Object object : listeners) {
            IBuildDefinitionStatusRecordListener listener = (IBuildDefinitionStatusRecordListener) object;
            listener.recordsReceived(records);
        }
    }

    /**
     * Get the count of listeners interested in build defintion status records.
     * 
     * @return The count of listeners interested in build defintion status
     *         records.
     */
    public int getBuildDefinitionListenerCount() {
        return fBuildDefinitionStatusRecordListeners.size();
    }

    /**
     * Get the count of listeners interested in build result records.
     * 
     * @return The count of listeners interested in build result records.
     */
    public int getBuildResultListenerCount() {
        return fBuildResultRecordListeners.size();
    }
}
