/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client;

import java.sql.Timestamp;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.client.ITeamBuildBaseClient;
import com.ibm.team.build.client.iterator.IBuildResultRecordIterator;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.common.ITeamBuildService;
import com.ibm.team.build.internal.common.model.dto.IBuildDefinitionStatusRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildEngineStatusRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildResultSearchCriteria;
import com.ibm.team.build.internal.common.model.dto.IBuildResultStatusTrend;
import com.ibm.team.build.internal.common.model.dto.IBuildScheduleRecord;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * The team build record interface provides access methods to access build
 * information in the form of DTO records.
 * 
 * @since 0.6
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface ITeamBuildRecordClient extends ITeamBuildBaseClient {

    /**
     * The maximum number of build result records that can be retrieved in one
     * invocation of
     * {@link #getBuildResultRecords(IBuildResultHandle[], String[], IProgressMonitor)}.
     */
    public static int MAX_BUILD_RESULT_RECORDS = ITeamBuildService.MAX_BUILD_RESULT_RECORDS;

    /**
     * Get the status records for the build definitions associated with the
     * given {@link IBuildDefinitionHandle}s.
     * 
     * @param handles
     *            {@link IllegalArgumentException} if
     *            {@link IBuildDefinitionHandle} is <code>null</code> or
     *            contains <code>null</code> values.
     * @param monitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * 
     * @return Array of {@link IBuildDefinitionStatusRecord}. May be empty, but
     *         never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(IBuildDefinitionHandle[] handles,
            IProgressMonitor monitor) throws TeamRepositoryException;

    /**
     * Get the status records for the specified build definitions.
     * 
     * @param buildDefinitionIds
     *            The ids of the build definitions.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of build definition status records. May be empty, but
     *         never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(final String[] buildDefinitionIds,
            IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Retrieves the engine status record for the specified build engine.
     * 
     * @param buildEngineHandle
     *            The build engine to retrieve the engine status record for.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return the engine status record for the specified build engine.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             if <code>buildEngineHandle</code> is null.
     */
    public IBuildEngineStatusRecord getBuildEngineStatusRecord(IBuildEngineHandle buildEngineHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Get the status records for all build definitions associated with the
     * given process areas.
     * 
     * @param processAreaHandles
     *            The process areas to search for associated build definitions.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of build definition status records. May be empty, but
     *         never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(IProcessAreaHandle[] processAreaHandles,
            IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Get the status records for the build definitions that are associated with
     * process areas to which the <code>contributor</code>belongs.
     * 
     * @param contributor
     *            {@link IContributor} whose build definitions this method
     *            returns.
     * @param connectedProjectAreas
     *            Array of {@link IProjectAreaHandle} that scope the search for
     *            process areas for the current user. If this is empty or
     *            <code>null</code>, all project areas will be searched for
     *            process areas to which the current user belongs.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of {@link IBuildDefinitionStatusRecord}. May be empty,
     *         but never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             if <code>contributor</code> is null.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(IContributor contributor,
            IProjectAreaHandle[] connectedProjectAreas, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Get the status records for the build definitions that are associated with
     * the given project areas.
     * 
     * @param projectAreas
     *            Array of {@link IProjectAreaHandle} that scope the search for
     *            build definitions. Must not be <code>null</code>.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of {@link IBuildDefinitionStatusRecord}. May be empty,
     *         but never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             if <code>projectAreas</code> is null.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(IProjectAreaHandle[] projectAreas,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Get the status records for all build engines.
     * 
     * @param projectAreaHandles
     *            The project areas to search for associated build engines. Must
     *            not be <code>null</code> or contain <code>null</code>
     *            elements.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of status records. May be empty, but never
     *         <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildEngineStatusRecord[] getBuildEngineStatusRecords(IProjectAreaHandle[] projectAreaHandles,
            IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Get build result records for the given handles.
     * 
     * @param resultHandles
     *            Array of {@link IBuildResultHandle},
     * @param properties
     *            The properties to include in the returned build result.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of build result records. Never <code>null</code>;
     *         will contain a null element for any resultHandle that could not
     *         be fetched.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the number of elements in <code>resultHandles</code>
     *             exceed {@link #MAX_BUILD_RESULT_RECORDS}.
     */
    public IBuildResultRecord[] getBuildResultRecords(IBuildResultHandle[] resultHandles, String[] properties,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;
    
    
    /**
     * Get the build result records iterator for all build results in the given
     * project area.
     * 
     * @param projectAreaHandle
     *            The project area to search for build results.
     * @param buildStates
     *            The build states to match. May be <code>null</code> or empty
     *            array to indicate all states.
     * @param properties
     *            The properties to include in the returned build results.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A build result record iterator for all build results in the given
     *         project area.. May be empty, but never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildResultRecordIterator getBuildResultRecords(IProjectAreaHandle projectAreaHandle,
            BuildState[] buildStates, String[] properties, IProgressMonitor progressMonitor)
            throws TeamRepositoryException;

    /**
     * Get the build result records iterator for all build results of the given
     * types.
     * 
     * @param buildDefinitionHandles
     *            The build definitions for which to get build results.
     * @param buildStates
     *            The build states to match. May be <code>null</code> or empty
     *            array to indicate all states.
     * @param properties
     *            The properties to include in the returned build results.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A build result record iterator for all build results of the given
     *         types. May be empty, but never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildResultRecordIterator getBuildResultRecords(IBuildDefinitionHandle[] buildDefinitionHandles,
            BuildState[] buildStates, String[] properties, IProgressMonitor progressMonitor)
            throws TeamRepositoryException;

    /**
     * Get the build result records iterator for all build results whose build
     * definitions are associated with the given process areas.
     * 
     * @param processAreaHandles
     *            The process areas to search for associated build
     *            definitions.
     * @param buildStates
     *            The build states to match. May be <code>null</code> or empty
     *            array to indicate all states.
     * @param properties
     *            The properties to include in the returned build results.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A build result records iterator for all build results whose build
     *         definitions are associated with the given process areas. May be
     *         empty, but never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildResultRecordIterator getBuildResultRecords(IProcessAreaHandle[] processAreaHandles,
            BuildState[] buildStates, String[] properties, IProgressMonitor progressMonitor)
            throws TeamRepositoryException;

    /**
     * Get the build result records iterator for all build results that match
     * the given build query criteria.
     * 
     * @param buildQueryCriteria
     *            The Build Query Criteria DTO to match with the desired build
     *            results
     * @param buildResultProperties
     *            The properties to include in the returned build results.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A build result records iterator for all build results that match
     *         the given build query criteria. May be empty, but never
     *         <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @since 0.8
     */
    public IBuildResultRecordIterator getBuildResultRecords(final IBuildResultSearchCriteria buildQueryCriteria,
            final String[] buildResultProperties, IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Get the build result record for the given build result.
     * 
     * @param buildResultHandle
     *            The build result.
     * @param properties
     *            The properties to include in the returned build result.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The build result record. Never <code>null</code>.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    public IBuildResultRecord getBuildResultRecord(IBuildResultHandle buildResultHandle, String[] properties,
            IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Get the status trend for the given build. The trend includes nearby build
     * status; if possible, some that precede the given build, and some that
     * follow it. The implementation attempts to provide a trend that includes
     * an even amount of previous and next builds, up to the
     * <code>maxResultCount</code>. Pending and canceled build results are
     * not included in the trend.
     * 
     * @param buildResultHandle
     *            The build result to get the trend for.
     * @param maxResultCount
     *            The maximum number of build results to include in the trend.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired. *
     * @return The status trend for the given build.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the specified <code>buildResultHandle</code> is
     *             <code>null</code> or <code>maxResultCount</code> exceeds
     *             {@link IBuildResultStatusTrend#MAX_TREND_LENGTH}.
     */
    public IBuildResultStatusTrend getBuildResultStatusTrend(IBuildResultHandle buildResultHandle, int maxResultCount,
            IProgressMonitor progressMonitor) throws TeamRepositoryException;

    /**
     * Get all of the available status trends for the given build. Each status
     * trend will have a type that corresponds to what the build results are
     * included in the trend. Two basic trends types are:
     * {@link IBuildResultStatusTrend#TYPE_ALL_VISIBLE}, which will not filter
     * out any build results, and
     * {@link IBuildResultStatusTrend#TYPE_NON_PERSONAL}, which will contain
     * only non-personal build results. The trends include nearby build status;
     * if possible, some that precede the given build, and some that follow it.
     * The implementation attempts to provide trends that include an even amount
     * of previous and next builds, up to the <code>maxResultCount</code>.
     * Pending and canceled build results are not included in the trend.
     * 
     * @param buildResultHandle
     *            The build result to get the trends for.
     * @param maxResultCount
     *            The maximum number of build results to include in the trends.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The status trends for the given build.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the specified <code>buildResultHandle</code> is 
     *             <code>null</code> or <code>maxResultCount</code> exceeds 
     *             {@link IBuildResultStatusTrend#MAX_TREND_LENGTH}.
     * @since 0.9.4
     */
    public IBuildResultStatusTrend[] getBuildResultStatusTrends(IBuildResultHandle buildResultHandle, int maxResultCount, 
            IProgressMonitor progressMonitor) throws TeamRepositoryException;
    
    /**
     * Get the build result records iterator for the build results that have the
     * given <code>tag</code> and whose definition is associated with a team
     * area in one of the given project areas.
     * 
     * @param tag
     *            {@link String} to use as the tag to search for. This should be
     *            a single tag, not a list of tags.
     * @param projectAreaHandles
     *            Array of {@link IProjectAreaHandle} that scope the search.
     * @param properties
     *            Array of {@link String} that determine what properties are on
     *            the returned {@link IBuildResultRecord}s.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A build result record iterator; may be empty if there are no
     *         builds with the given tag.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If ANY of the following is true:
     *             <ul>
     *             <li><code>projectAreaHandles</code> is <code>null</code></li>
     *             <li><code>projectAreaHandles</code> contains
     *             <code>null</code> slots</li>
     *             <li><code>properties</code> contains <code>null</code>
     *             slots </li>
     *             <li><code>tag</code> is <code>null</code>.</li>
     *             <li><code>tag</code> is the empty string.</li>
     *             <li><code>tag</code> contains spaces or commas. The spaces
     *             and commas are commonly used to denote multiple tags, and
     *             this method does not allow multiple tags.</li>
     */
    public IBuildResultRecordIterator getBuildResultRecords(String tag, IProjectAreaHandle[] projectAreaHandles,
            String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Get the build result records iterator for the build results that have the
     * given <code>tag</code> and are for one of the given build definitions;
     * if <code>definitionHandles</code> is <code>null</code> or empty, this
     * method finds all results with the specified tag.
     * 
     * @param definitionHandles
     *            Array of {@link IBuildDefinitionHandle} to use as a filter for
     *            results. May be <code>null</code> or empty to find all
     *            definitions in the given category.
     * @param tag
     *            {@link String} the tag to search for. May not be
     *            <code>null</code>.
     * @param properties
     *            The properties of the build result to retrieve.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return A build result records iterator for build results that are for
     *         one of the the given definitions and are tagged with the given
     *         tag.
     * @throws TeamRepositoryException
     *             if there is a problem reading from the repository
     * @throws IllegalArgumentException
     *             If ANY of the following is true:
     *             <ul>
     *             <li><code>properties</code> contains any <code>null</code>
     *             slots </li>
     *             <li><code>tag</code> is <code>null</code>.</li>
     *             <li><code>tag</code> is the empty string.</li>
     *             <li><code>tag</code> contains spaces or commas. The spaces
     *             and commas are commonly used to denote multiple tags, and
     *             this method does not allow multiple tags.</li>
     */
    public IBuildResultRecordIterator getBuildResultRecords(String tag, IBuildDefinitionHandle[] definitionHandles,
            String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException;

    /**
     * Get the build schedule record for the given build definition.
     * 
     * @param buildDefinitionHandle
     *            The build definition.
     * @param timeZoneIDs
     *            The IDs of the time zones to convert to. Use a
     *            <code>null</code> to return schedule entries as configured, in
     *            which case the record's time zone is set to the server time
     *            zone.
     * @param dstAdjustmentDate
     *            If non-<code>null</code>, daylight savings time adjustments
     *            are taken into account when converting between time zones,
     *            using the DST offsets in effect on the given date/time. DST is
     *            not included when returning the as-configured entries.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return the build schedule records for the given time zones.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If <code>buildDefinitionHandle</code> or
     *             <code>timeZoneIDs</code> is <code>null</code>.
     * @since 10.0.2
     */
    public IBuildScheduleRecord[] getBuildScheduleRecords(IBuildDefinitionHandle buildDefinitionHandle,
            String[] timeZoneIDs, Timestamp dstAdjustmentDate, IProgressMonitor progressMonitor)
            throws TeamRepositoryException;

    /**
     * Saves the build definition and updates the specified build engines to
     * support the definition according to the specified
     * <code>isSupported</code> flags. Also saves any new engines that are
     * specified in <code>newBuildEngines</code>.
     * 
     * @param buildDefinition
     *            The build definition to save.
     * @param buildEngineHandles
     *            The build engines to update to support the build definition.
     * @param isSupported
     *            Indicates if the build engine should be supported. Any value
     *            other than <code>0</code> is considered true.
     * @param newBuildEngines
     *            The new build engines that should be saved.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return An array of all items that were saved. The first item in the
     *         array will always be the build definition that was saved.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     * @throws IllegalArgumentException
     *             If the <code>buildDefinition</code> is <code>null</code>
     *             or any of the other array parameters contain a
     *             <code>null</code> element.
     */
    public IItem[] save(IBuildDefinition buildDefinition, IBuildEngineHandle[] buildEngineHandles, int[] isSupported,
            IBuildEngine[] newBuildEngines, IProgressMonitor progressMonitor) throws TeamRepositoryException;

}
