/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.regex.PatternSyntaxException;

import com.ibm.team.build.client.ITeamBuildClient;

/**
 * Helper class for dealing with proxy host and port configuration.
 */
public class ProxyHelper {

    protected static final String HTTP_PROTOCOL = "http:"; //$NON-NLS-1$
    protected static final String HTTPS_PROTOCOL = "https:"; //$NON-NLS-1$
    protected static final String NONPROXYHOST_SEPARATOR = "\\|"; //$NON-NLS-1$
    protected static final String WILDCARD_CHARACTER = "*"; //$NON-NLS-1$
    /**
     * Get the proxy host as configured by the http/https proxy properties
     * defined in {@link ITeamBuildClient}.
     * 
     * @param repositoryAddress
     *            The repository address. The address is used only for examining
     *            the protocol to determine whether it is http or https.
     * @return The proxy host or <code>null</code> if a proxy host was not
     *         configured for the protocol or null if the repository address is part
     *         of no proxy hosts
     *         
     */
    public static String getProxyHost(String repositoryAddress) {
        if (isANoProxyHost(repositoryAddress)) {
            return null;
        }
        if (repositoryAddress.startsWith(HTTPS_PROTOCOL)) {
            return System.getProperty(ITeamBuildClient.BUILD_CLIENT_HTTPS_PROXY_HOST_PROPERTY);
        }
        return System.getProperty(ITeamBuildClient.BUILD_CLIENT_HTTP_PROXY_HOST_PROPERTY);
    }

    /**
     * Returns the scheme specified by the referenced <code>repositoryAddress</code>.
     *  
     * @param repositoryAddress
     *            The repository address. The address is used only for examining
     *            the protocol to determine whether it is http or https. Must not be <code>null</code>.
     * @return The repositoryAddress scheme.  Will be "http", "https" or <code>null</code> if the scheme portion
     *         of the repositoryAddress is unrecognized.
     */
    public static String getProxyType(String repositoryAddress) {
        if (repositoryAddress.startsWith(HTTPS_PROTOCOL))
            return "https"; //$NON-NLS-1$
        else if (repositoryAddress.startsWith(HTTP_PROTOCOL))
            return "http"; //$NON-NLS-1$
        else
            return null;
    }
    
    /**
     * Get the proxy port as configured by the http/https proxy properties
     * defined in {@link ITeamBuildClient}.
     * 
     * @param repositoryAddress
     *            The repository address. The address is used only for examining
     *            the protocol to determine whether it is http or https.
     * @return The proxy port. If a proxy port was not configured for the
     *         protocol, a default is returned: 80 for http, or 443 for https.
     *         The result of this method should not be used if
     *         <code>getProxyHost</code> returns <code>null</code>.
     */
    public static int getProxyPort(String repositoryAddress) {
        int port = 0;

        String portString = null;

        if (repositoryAddress.startsWith(HTTPS_PROTOCOL)) {
            portString = System.getProperty(ITeamBuildClient.BUILD_CLIENT_HTTPS_PROXY_PORT_PROPERTY,
                    ITeamBuildClient.BUILD_CLIENT_HTTPS_PROXY_PORT_DEFAULT);
        } else {
            portString = System.getProperty(ITeamBuildClient.BUILD_CLIENT_HTTP_PROXY_PORT_PROPERTY,
                    ITeamBuildClient.BUILD_CLIENT_HTTP_PROXY_PORT_DEFAULT);
        }

        try {
            port = Integer.parseInt(portString);
        } catch (NumberFormatException exception) {
            // Just ignore it.
        }
        return port;
    }

    /**
     * Get the proxy user name as configured by the http/https proxy properties
     * defined in {@link ITeamBuildClient}.
     * 
     * @param repositoryAddress
     *            The repository address. The address is used only for examining
     *            the protocol to determine whether it is http or https.
     * @param defaultUsername
     *            This will be the default string to use if the proxy user is
     *            not supplied using Java system properties (usually the jazz
     *            user name). Can be <code>null</code> if no default user name
     *            is to be used.
     * @return The string to use as the proxy username. This value is calculated
     *         from the Java system properties <code>https.proxyUser</code> or
     *         <code>http.proxyUser</code>. If the user name is not specified
     *         using any Java system properties and <code>defaultUsername</code>
     *         is not <code>null</code> then <code>defaultUsername</code> will
     *         be returned instead. A value of <code>null</code> will be
     *         returned if the Java system property is defined and the value is
     *         empty, or if the user name is not specified using Java system
     *         properties and the <code>defaultUsername</code> is
     *         <code>null</code>.
     * 
     * @since 0.8.1
     */
    public static String getProxyUser(String repositoryAddress, String defaultUsername) {
        String proxyUser = defaultUsername;
        String propertyKey = ITeamBuildClient.BUILD_CLIENT_HTTPS_PROXY_USER_PROPERTY;
        if (repositoryAddress.startsWith(HTTP_PROTOCOL)) {
            propertyKey = ITeamBuildClient.BUILD_CLIENT_HTTP_PROXY_USER_PROPERTY;
        }

        if (defaultUsername != null) {
            proxyUser = System.getProperty(propertyKey, defaultUsername);
        } else {
            proxyUser = System.getProperty(propertyKey);
        }

        if (proxyUser != null && proxyUser.length() == 0) {
            proxyUser = null;
        }

        return proxyUser;
    }

    /**
     * Get the proxy password as configured by the http/https proxy properties
     * defined in {@link ITeamBuildClient}.
     * 
     * @param repositoryAddress
     *            The repository address. The address is used only for examining
     *            the protocol to determine whether it is http or https.
     * @param defaultPassword
     *            This will be the default string to use if the proxy password
     *            is not supplied using Java system properties. (usually the
     *            Jazz password)
     * @return The string to use as the proxy password. This value is calculated
     *         from the Java system properties <code>https.proxyPassword</code>
     *         or <code>http.proxyPassword</code>. If the password is not
     *         specified using any Java system properties and
     *         <code>defaultPassword</code> is not <code>null</code> then
     *         <code>defaultPassword</code> will be returned instead. A value of
     *         <code>null</code> will be returned if the Java system property is
     *         defined and the value is empty, or if the password is not
     *         specified using Java system properties and the
     *         <code>defaultUsername</code> is <code>null</code>.
     * 
     * @since 0.8.1
     */
    public static String getProxyPassword(String repositoryAddress, String defaultPassword) {
        String proxyPassword = defaultPassword;
        String propertyKey = ITeamBuildClient.BUILD_CLIENT_HTTPS_PROXY_PASSWORD_PROPERTY;
        if (repositoryAddress.startsWith(HTTP_PROTOCOL)) {
            propertyKey = ITeamBuildClient.BUILD_CLIENT_HTTP_PROXY_PASSWORD_PROPERTY;
        }

        if (defaultPassword != null) {
            proxyPassword = System.getProperty(propertyKey, defaultPassword);
        } else {
            proxyPassword = System.getProperty(propertyKey);
        }

        if (proxyPassword != null && proxyPassword.length() == 0) {
            proxyPassword = null;
        }

        return proxyPassword;
    }
    
    /**
     * Return no proxy hosts for the given repository address.
     * @param repositoryAddress
     * @return a comma separated list of no proxy hosts or <code>null</code> if it is undefined or empty string
     */
    public static String getNoProxyHosts(String repositoryAddress) {
        String noProxyHost = null;
        if (repositoryAddress.startsWith(HTTPS_PROTOCOL)) {
            noProxyHost = System.getProperty(ITeamBuildClient.BUILD_CLIENT_HTTPS_NO_PROXY_HOST_PROPERTY);
        }
        else {
            noProxyHost = System.getProperty(ITeamBuildClient.BUILD_CLIENT_HTTP_NO_PROXY_HOST_PROPERTY);
        }
        if (noProxyHost != null) {
            noProxyHost = noProxyHost.trim();
            if (noProxyHost.isEmpty()) {
                noProxyHost = null;
            }
        }
        return noProxyHost;
    }
    
    /**
     * Check whether the given repository address is to be excluded from proxy configuration.
     * @param repositoryAddress
     * @return true if no proxy has to be applied to this address, false otherwise
     */
    public static boolean isANoProxyHost(String repositoryAddress) {
        String noProxyHostsStr = getNoProxyHosts(repositoryAddress);
        
        // If no proxy hosts is not defined, then we have to contact the proxy
        if (noProxyHostsStr == null) return false;

        URI repoURI = null;
        try {
            repoURI = new URI(repositoryAddress);
        }
        catch (URISyntaxException exp) {
            return false;
        }
        String host = repoURI.getHost();
        String [] noProxyHosts = noProxyHostsStr.split(NONPROXYHOST_SEPARATOR);        
        for (String noProxyHost : noProxyHosts) {
            noProxyHost = noProxyHost.trim();
            int noProxyHostLen = noProxyHost.length();
            /* Three cases to handle
             * If noProxyHost is an empty string, then continue
             * If noProxyHost has a pattern character, then 
             *    create a pattern out of the noProxyHost (escape . character which has special meaning in regex)
             *    match the pattern against the given host name.
             * If noProxyHost length matches our host length or noProxyHost length is less than host length and starts with a . 
             * then do a suffix match by aligning the end of two strings and comparing towards the left. 
             * Else, continue to the next noProxyHost
             */ 
            if (noProxyHostLen == 0) {
                continue;
            }
            else if (noProxyHost.contains(WILDCARD_CHARACTER)) {
                // Escape the . character in host name because it needs to be matched literally.
                String escapedNoProxyHost = noProxyHost.replace(".", "\\."); //$NON-NLS-1$ //$NON-NLS-2$
                // * has to be prepended by . to ensure that it matches any character, not just the character preceding * 
                escapedNoProxyHost = escapedNoProxyHost.replace("*", ".*");  //$NON-NLS-1$ //$NON-NLS-2$
                try {
                    if (host.matches(escapedNoProxyHost)) {
                        return true;
                    }
                }
                catch (PatternSyntaxException exp) {
                    // TODO Log the issue
                    // Ignore and continue to the next nonproxy host
                    continue;
                }
            }
            else if ( (noProxyHostLen == host.length() || (noProxyHost.length() < host.length() &&  noProxyHost.startsWith("."))) //$NON-NLS-1$ 
                  && 
                  host.regionMatches(true,  host.length() - noProxyHostLen, noProxyHost, 0,  noProxyHostLen)) { 
                    return true;
            }
            else {
                // fall through to the next noProxyHost
                continue;
            }
        }
        return false;
        
    }
}
