/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.client;

import java.sql.Timestamp;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.IBuildActivity;
import com.ibm.team.build.common.model.IBuildActivityId;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.common.model.query.IBaseBuildResultQueryModel.IBuildResultQueryModel;
import com.ibm.team.build.internal.common.ITeamBuildService;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.common.model.dto.IBuildItemChangeResponse;
import com.ibm.team.build.internal.common.model.dto.IBuildItemNamePair;
import com.ibm.team.process.client.IProcessClientService;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.advice.IItemsResponse;
import com.ibm.team.process.common.advice.IObjectsResponse;
import com.ibm.team.process.common.advice.IOperationReport;
import com.ibm.team.process.common.advice.ProcessRunnable;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.util.IClientLibraryContext;
import com.ibm.team.repository.client.util.IClientLibraryContext.IServiceRunnable;
import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.query.IItemQuery;
import com.ibm.team.repository.common.query.IItemQueryPage;
import com.ibm.team.repository.common.query.ast.IPredicate;

/**
 * The client library implementation for the team build service.
 */
public class TeamBuildClient extends AbstractTeamBuildClient implements ITeamBuildClient {

    /**
     * Create the team build client.
     * 
     * @param context
     *            The context to create the abstract team build client for.
     */
    public TeamBuildClient(IClientLibraryContext context) {
        super(context);
    }

    private ITeamBuildService getService() {
        return (ITeamBuildService) getContext().getServiceInterface(ITeamBuildService.class);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildDefinition getBuildDefinition(final String buildDefinitionId, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        
        ValidationHelper.validateNotNull("buildDefinitionId", buildDefinitionId); //$NON-NLS-1$

        return (IBuildDefinition) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return addToItemManager(getService().getBuildDefinition(buildDefinitionId));
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public Timestamp getLastContactTime(final IBuildEngineHandle buildEngineHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return (Timestamp) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getLastContactTime(buildEngineHandle);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public long getTimeSinceLastContact(final IBuildEngineHandle buildEngineHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildEngineHandle", buildEngineHandle); //$NON-NLS-1$

        return (Long) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getTimeSinceLastContact(buildEngineHandle);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildEngine setLastContactTime(final IBuildEngineHandle buildEngineHandle, final Timestamp lastContactTime,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildEngineHandle", buildEngineHandle); //$NON-NLS-1$

        class LastContactRunnable extends ProcessRunnable {

            public IBuildEngine fBuildEngine;

            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().setLastContactTime(buildEngineHandle, lastContactTime);
                fBuildEngine = (IBuildEngine) response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildEngine) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                LastContactRunnable runnable = new LastContactRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildClient_SET_LAST_CONTACT_TIME_OPERATION_NAME,
                        progressMonitor);

                return (IBuildEngine) addToItemManager(runnable.fBuildEngine);
            }
        });

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildEngine getBuildEngine(final String id, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        
        ValidationHelper.validateNotNull("id", id); //$NON-NLS-1$

        return (IBuildEngine) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return addToItemManager(getService().getBuildEngine(id));
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultContribution[] getBuildResultContributions(IBuildResultHandle buildResultHandle,
            String extendedContributionTypeId, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException {
        return getBuildResultContributions(buildResultHandle, new String[] { extendedContributionTypeId },
                progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultContribution[] getBuildResultContributions(final IBuildResultHandle buildResultHandle,
            final String[] extendedContributionTypeIds, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("extendedContributionTypeIds", extendedContributionTypeIds); //$NON-NLS-1$
        
        return (IBuildResultContribution[]) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getBuildResultContributions(buildResultHandle, extendedContributionTypeIds);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void deleteBuildResultContributions(final IBuildResultHandle buildResultHandle,
            final IBuildResultContribution[] buildResultContributions, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("buildResultContributions", buildResultContributions); //$NON-NLS-1$

        callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor cancelableMonitor) throws TeamRepositoryException {
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(new ProcessRunnable() {

                    @Override
                    public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                        return getService().deleteBuildResultContributions(buildResultHandle, buildResultContributions);
                    }
                }, Messages.AbstractTeamBuildClient_SAVE_BUILD_RESULT_OPERATION_NAME, progressMonitor);
                return null;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResult addBuildResultContributions(final IBuildResultHandle buildResultHandle,
            final IBuildResultContribution[] buildResultContributions, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("buildResultContributions", buildResultContributions); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class AddContributionsRunnable extends ProcessRunnable {

            public IItem fBuildResult;

            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().addBuildResultContributions(buildResultHandle,
                        buildResultContributions, properties);
                fBuildResult = response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildResult) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                AddContributionsRunnable runnable = new AddContributionsRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.AbstractTeamBuildClient_SAVE_BUILD_RESULT_OPERATION_NAME,
                        progressMonitor);

                return addToItemManager(runnable.fBuildResult);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResult addBuildResultContribution(IBuildResultHandle buildResultHandle,
            IBuildResultContribution buildResultContribution, String[] properties, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        return addBuildResultContributions(buildResultHandle,
                new IBuildResultContribution[] { buildResultContribution }, properties, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void addBuildResultContributions(IBuildResultHandle buildResultHandle,
            IBuildResultContribution[] buildResultContributions, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        addBuildResultContributions(buildResultHandle, buildResultContributions, IBuildResult.PROPERTIES_REQUIRED,
                progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void addBuildResultContribution(IBuildResultHandle buildResultHandle,
            IBuildResultContribution buildResultContribution, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        addBuildResultContributions(buildResultHandle, new IBuildResultContribution[] { buildResultContribution },
                IBuildResult.PROPERTIES_REQUIRED, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String startBuildActivity(final IBuildResultHandle buildResultHandle, final String label,
            final String parentId, final boolean autoComplete, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("label", label); //$NON-NLS-1$

        class StartBuildActivityRunnable extends ProcessRunnable {

            public String fActivityId;

            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IObjectsResponse response = getService().startBuildActivity(buildResultHandle, label, parentId,
                        autoComplete);
                IBuildActivityId activityId = (IBuildActivityId) response.getClientObjects()[0];
                fActivityId = activityId.getId();
                return response.getOperationReport();
            }
        }

        return (String) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                StartBuildActivityRunnable runnable = new StartBuildActivityRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.AbstractTeamBuildClient_START_BUILD_ACTIVITY_OPERATION_NAME,
                        monitor);
                return runnable.fActivityId;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void completeBuildActivity(final IBuildResultHandle buildResultHandle, final String activityId,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("activityId", activityId); //$NON-NLS-1$

        class EndBuildActivityRunnable extends ProcessRunnable {
            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().completeBuildActivity(buildResultHandle, activityId);
            }

        }
        callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                EndBuildActivityRunnable runnable = new EndBuildActivityRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.AbstractTeamBuildClient_END_BUILD_ACTIVITY_OPERATION_NAME,
                        monitor);
                return null;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildActivity[] getBuildActivities(final IBuildResultHandle buildResultHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$

        return (IBuildActivity[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildActivity[] allActivities = getService().getBuildActivities(buildResultHandle);

                /*
                 * The array of activities we get from the server is a flat list
                 * of activities - there is no hierarchical representation of
                 * child activities. So we have to create the hierarchy here by
                 * populating the child activities of all activities that have
                 * children, then returning an array of only the root
                 * activities.
                 */

                /*
                 * First, create a mapping of activity id to activity object for
                 * convenience.
                 */
                Map<String, IBuildActivity> activityMap = new Hashtable<String, IBuildActivity>();
                for (IBuildActivity activity : allActivities) {
                    activityMap.put(activity.getUniqueId(), activity);
                }

                /*
                 * Now create a list of all child ids...
                 */
                List<String> childActivityIds = new LinkedList<String>();

                for (IBuildActivity activity : allActivities) {
                    for (IBuildActivityId childId : activity.getChildActivityIds()) {
                        childActivityIds.add(childId.getId());
                    }
                }

                /*
                 * ...then add all non-child activities to a list of "root"
                 * activities.
                 */
                List<IBuildActivity> rootActivityList = new LinkedList<IBuildActivity>();

                for (IBuildActivity activity : allActivities) {
                    if (!childActivityIds.contains(activity.getUniqueId())) {
                        rootActivityList.add(activity);
                    }
                }

                /*
                 * finally, for each activity in the root activity list,
                 * recursively populate its list of child activities
                 */
                for (IBuildActivity activity : rootActivityList) {
                    populateChildActivities(activity, activityMap);
                }

                return rootActivityList.toArray(new IBuildActivity[rootActivityList.size()]);
            }
        });
    }

    private void populateChildActivities(IBuildActivity parentActivity, Map<String, IBuildActivity> allActivities) {
        for (IBuildActivityId childActivityId : parentActivity.getChildActivityIds()) {
            IBuildActivity childActivity = allActivities.get(childActivityId.getId());
            parentActivity.addChildActivity(childActivity);
            populateChildActivities(childActivity, allActivities);
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String[] getTags(final IProjectAreaHandle[] projectAreaHandles, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("projectAreaHandles", projectAreaHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("projectAreaHandles", projectAreaHandles); //$NON-NLS-1$

        return (String[]) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getTags(projectAreaHandles);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String[] getTags(final IBuildDefinitionHandle[] definitionHandles, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNullElements("definitionHandles", definitionHandles); //$NON-NLS-1$

        return (String[]) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getTagsForBuildDefinitions(definitionHandles);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getInProgressActivityLabel(IBuildResultHandle buildResultHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        ITeamBuildRecordClient recordClient = (ITeamBuildRecordClient) getRepository().getClientLibrary(
                ITeamBuildRecordClient.class);
        return recordClient.getBuildResultRecord(buildResultHandle, IBuildResult.PROPERTIES_COMPLETE, progressMonitor).getCurrentActivityLabel();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public int getPercentComplete(IBuildResultHandle buildResultHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        ITeamBuildRecordClient recordClient = (ITeamBuildRecordClient) getRepository().getClientLibrary(
                ITeamBuildRecordClient.class);
        return recordClient.getBuildResultRecord(buildResultHandle, IBuildResult.PROPERTIES_COMPLETE, progressMonitor).getPercentComplete();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResult getLastBuildResult(final IBuildDefinitionHandle buildDefinitionHandle, BuildState[] buildStates,
            String[] properties, final IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException {

        ValidationHelper.validateNotNull("buildDefinitionHandle", buildDefinitionHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildStates", buildStates); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        IBuildResultQueryModel buildResultQueryModel = IBuildResultQueryModel.ROOT;
        final IItemQuery query = IItemQuery.FACTORY.newInstance(buildResultQueryModel);
        IPredicate left = buildResultQueryModel.buildDefinition()._eq(query.newItemHandleArg());
        IPredicate right = buildResultQueryModel.buildState()._in(getStateNames(buildStates));
        query.filter(left._and(right));
        query.orderByDsc(buildResultQueryModel.buildStartTime());
        query.setResultLimit(1);

        IItemQueryPage queryPage = (IItemQueryPage) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return queryItems(query, new Object[] { buildDefinitionHandle }, 1, progressMonitor);
            }
        });

        IBuildResult buildResult = null;
        if (queryPage.getSize() > 0) {
            IItemHandle buildResultHandle = queryPage.handlesAsArray()[0];
            if (properties == IBuildResult.PROPERTIES_COMPLETE) {
                buildResult = (IBuildResult) getRepository().itemManager().fetchCompleteItem(buildResultHandle,
                        IItemManager.DEFAULT, progressMonitor);
            } else {
                buildResult = (IBuildResult) getRepository().itemManager().fetchPartialItem(buildResultHandle,
                        IItemManager.DEFAULT, toCollection(properties), progressMonitor);
            }
        }

        return buildResult;
    }

    /* (non-Javadoc)
     * Intentionally not documented. See parent.
     */
    public boolean anyResponsiveBuildEngines(final IBuildDefinitionHandle buildDefinitionHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildDefinitionHandle", buildDefinitionHandle); //$NON-NLS-1$

        return (Boolean) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return new Boolean(getService().anyResponsiveBuildEngines(buildDefinitionHandle));
            }
        });
    }

    /**
     * Retrieves the build state names for the specified build states.
     * 
     * @param buildStates
     *            The build states to convert to state names.
     */
    private String[] getStateNames(BuildState[] buildStates) {
        String[] buildStateNames = new String[buildStates.length];
        for (int i = 0; i < buildStates.length; i++) {
            buildStateNames[i] = buildStates[i].name();
        }
        return buildStateNames;
    }

    /* (non-Javadoc)
     * Intentionally not documented. See parent.
     */
    public boolean anyBuildEnginesWithRequestProcessingEnabled(final IBuildDefinitionHandle buildDefinitionHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildDefinitionHandle", buildDefinitionHandle); //$NON-NLS-1$

        return (Boolean) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return new Boolean(getService().anyBuildEnginesWithRequestProcessingEnabled(buildDefinitionHandle));
            }
        });
    }
    
    @Override
    public void moveItemsToFolder(final IItemHandle[] itemHandles, final IBuildFolderHandle folderHandle,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        ValidationHelper.validateElementsAreOneOfTypes("itemHandles", itemHandles, new Class[] { IBuildFolderHandle.class, IBuildDefinitionHandle.class } ); //$NON-NLS-1$
        callCancelableService(progressMonitor, new IServiceRunnable() {
            @Override
            public Object run(IProgressMonitor cancelableMonitor) throws TeamRepositoryException {
                    IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(IProcessClientService.class);
                    return processService.execute(new ProcessRunnable() {
                        @Override
                        public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                            IBuildItemChangeResponse response = getService().moveBuildItemsToFolder(itemHandles, folderHandle);
                            
                            addToItemManager(response.getDeletedItems(), progressMonitor);
                            deleteFromItemManager(response.getDeletedItems());
                            addToItemManager(response.getAddedItems(), progressMonitor);
                            addToItemManager(response.getUpdatedItems(), progressMonitor);
                            return null;
                        }
                    }, Messages.AbstractTeamBuildClient_MOVE_BUILD_ITEMS_TO_FOLDER_OPERATION_NAME, progressMonitor);
            }
 
        });
        
        
        
    }

    @Override
    public IBuildItemNamePair[] getChildrenOfFolder(final IBuildFolderHandle buildViewFolderHandle, final IProcessAreaHandle[] processAreaHandles, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        ValidationHelper.validateNotNull("processAreaHandles", processAreaHandles); //$NON-NLS-1$
        return (IBuildItemNamePair[]) callCancelableService(progressMonitor, new IServiceRunnable() {
            @Override
            public Object run(IProgressMonitor cancelableMonitor) throws TeamRepositoryException {
                IBuildItemNamePair[] buildItems = getService().getChildrenOfBuildFolder(buildViewFolderHandle, processAreaHandles);
                return buildItems;
            }
        });
    }
    
    public IBuildItemNamePair[] getAncestorFolders(final IItemHandle itemHandle, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        ValidationHelper.validateNotNull("itemHandle", itemHandle); //$NON-NLS-1$
        
        return (IBuildItemNamePair[]) callCancelableService(progressMonitor, new IServiceRunnable<IBuildItemNamePair[]>() {
            @Override
            public IBuildItemNamePair[] run(IProgressMonitor cancelableMonitor) throws TeamRepositoryException {
                return getService().getAncestorFolders(itemHandle);
            }
        });
    }
}
