/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.client.iterator.IBuildResultRecordIterator;
import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.IBuildAverageData;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.client.iterator.BuildResultRecordIterator;
import com.ibm.team.build.internal.common.ITeamBuildService;
import com.ibm.team.build.internal.common.helper.TagsHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.common.model.dto.BuildDefinitionStatusRecord;
import com.ibm.team.build.internal.common.model.dto.BuildEngineStatusRecord;
import com.ibm.team.build.internal.common.model.dto.BuildResultRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildDefinitionStatusRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildEngineStatusRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.build.internal.common.model.dto.IBuildResultSearchCriteria;
import com.ibm.team.build.internal.common.model.dto.IBuildResultStatusTrend;
import com.ibm.team.build.internal.common.model.dto.IBuildScheduleRecord;
import com.ibm.team.build.internal.common.model.dto.IInProgressBuild;
import com.ibm.team.build.internal.common.model.dto.InProgressBuild;
import com.ibm.team.build.internal.common.model.dto.impl.BuildDefinitionStatusRecordImpl;
import com.ibm.team.build.internal.common.model.dto.impl.BuildEngineStatusRecordImpl;
import com.ibm.team.build.internal.common.model.dto.impl.BuildResultRecordImpl;
import com.ibm.team.build.internal.common.model.dto.impl.InProgressBuildImpl;
import com.ibm.team.process.client.IProcessClientService;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.advice.IItemsResponse;
import com.ibm.team.process.common.advice.IOperationReport;
import com.ibm.team.process.common.advice.ProcessRunnable;
import com.ibm.team.repository.client.util.IClientLibraryContext;
import com.ibm.team.repository.client.util.IClientLibraryContext.IServiceRunnable;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.query.IItemQueryPage;

/**
 * The client library implementation for the team build service DTO record
 * methods.
 */
public class TeamBuildRecordClient extends AbstractTeamBuildClient implements ITeamBuildRecordClient {

    /**
     * Create the team build record client.
     * 
     * @param context
     *            The context to create the abstract team build record client
     *            for.
     */
    public TeamBuildRecordClient(IClientLibraryContext context) {
        super(context);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(final IBuildDefinitionHandle[] handles,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {
        ValidationHelper.validateNotNull("handles", handles); //$NON-NLS-1$

        return (IBuildDefinitionStatusRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildDefinitionStatusRecord[] records = getService().getBuildDefinitionStatusRecordsForDefinitions(handles);
                for (IBuildDefinitionStatusRecord record : records) {
                    processBuildDefinitionStatusRecord(record);
                }

                BuildRecordEventManager.getInstance().notifyRecordsReceived(records);
                return records;
            }
        });

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(final String[] buildDefinitionIds,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("buildDefinitionIds", buildDefinitionIds); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("buildDefinitionIds", buildDefinitionIds); //$NON-NLS-1$

        return (IBuildDefinitionStatusRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildDefinitionStatusRecord[] records = getService().getBuildDefinitionStatusRecordsById(
                        buildDefinitionIds);
                for (IBuildDefinitionStatusRecord record : records) {
                    processBuildDefinitionStatusRecord(record);
                }

                BuildRecordEventManager.getInstance().notifyRecordsReceived(records);
                return records;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(final IProcessAreaHandle[] processAreaHandles,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("processAreaHandles", processAreaHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("processAreaHandles", processAreaHandles); //$NON-NLS-1$

        return (IBuildDefinitionStatusRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildDefinitionStatusRecord[] records = getService().getBuildDefinitionStatusRecords(processAreaHandles);
                for (IBuildDefinitionStatusRecord record : records) {
                    processBuildDefinitionStatusRecord(record);
                }

                BuildRecordEventManager.getInstance().notifyRecordsReceived(records);
                return records;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(final IContributor contributor,
            final IProjectAreaHandle[] connectedProjectAreas, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        
        ValidationHelper.validateNotNull("contributor", contributor); //$NON-NLS-1$

        return (IBuildDefinitionStatusRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildDefinitionStatusRecord[] records = getService().getBuildDefinitionStatusRecordsForContributor(
                        contributor, connectedProjectAreas);
                for (IBuildDefinitionStatusRecord record : records) {
                    processBuildDefinitionStatusRecord(record);
                }

                BuildRecordEventManager.getInstance().notifyRecordsReceived(records);
                return records;
            }
        });

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildDefinitionStatusRecord[] getBuildDefinitionStatusRecords(final IProjectAreaHandle[] projectAreas,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("projectAreas", projectAreas); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("projectAreas", projectAreas); //$NON-NLS-1$

        return (IBuildDefinitionStatusRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildDefinitionStatusRecord[] records = getService().getBuildDefinitionStatusRecordsForProjectAreas(
                        projectAreas);
                for (IBuildDefinitionStatusRecord record : records) {
                    processBuildDefinitionStatusRecord(record);
                }

                BuildRecordEventManager.getInstance().notifyRecordsReceived(records);
                return records;
            }
        });

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildEngineStatusRecord[] getBuildEngineStatusRecords(final IProjectAreaHandle[] projectAreaHandles,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("projectAreaHandles", projectAreaHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("projectAreaHandles", projectAreaHandles); //$NON-NLS-1$

        return (IBuildEngineStatusRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildEngineStatusRecord[] records = getService().getBuildEngineStatusRecords(projectAreaHandles);
                for (IBuildEngineStatusRecord record : records) {
                    processBuildEngineStatusRecord(record);
                }
                return records;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildEngineStatusRecord getBuildEngineStatusRecord(final IBuildEngineHandle buildEngineHandle,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("buildEngineHandle", buildEngineHandle); //$NON-NLS-1$

        return (IBuildEngineStatusRecord) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildEngineStatusRecord record = getService().getBuildEngineStatusRecord(buildEngineHandle);
                processBuildEngineStatusRecord(record);
                return record;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecord[] getBuildResultRecords(final IBuildResultHandle[] resultHandles,
            final String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException {
        ValidationHelper.validateNotNull("resultHandles", resultHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("resultHandles", resultHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        return (IBuildResultRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IBuildResultRecord[] records = getService().getBuildResultRecordsForBuildResults(resultHandles,
                        properties);
                
                List<IBuildResultRecord> nonNullRecords = new LinkedList<IBuildResultRecord>();
                
                for (IBuildResultRecord record : records) {
                    if (record != null) {
                        processBuildResultRecord(record);
                        nonNullRecords.add(record);
                    }
                }

                BuildRecordEventManager.getInstance().notifyRecordsReceived(
                        nonNullRecords.toArray(new IBuildResultRecord[nonNullRecords.size()]));

                return records;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecordIterator getBuildResultRecords(final IProjectAreaHandle projectAreaHandle,
            final BuildState[] buildStates, final String[] properties, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException {

        ValidationHelper.validateNotNull("projectAreaHandle", projectAreaHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        return (IBuildResultRecordIterator) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {

                IItemQueryPage queryPage = getService().getQueryPageForBuildResultsForProjectArea(projectAreaHandle,
                        getBuildStateNames(buildStates));
                return new BuildResultRecordIterator(getRepository(), queryPage, properties);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecordIterator getBuildResultRecords(final IBuildDefinitionHandle[] buildDefinitionHandles,
            final BuildState[] buildStates, final String[] properties, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        ValidationHelper.validateNotNull("buildDefinitionHandles", buildDefinitionHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("buildDefinitionHandles", buildDefinitionHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        return (IBuildResultRecordIterator) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemQueryPage queryPage = getService().getQueryPageForBuildResultsForBuildDefinitions(
                        buildDefinitionHandles, getBuildStateNames(buildStates));

                return new BuildResultRecordIterator(getRepository(), queryPage, properties);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecordIterator getBuildResultRecords(final IProcessAreaHandle[] processAreaHandles,
            final BuildState[] buildStates, final String[] properties, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException {

        ValidationHelper.validateNotNull("processAreaHandles", processAreaHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("processAreaHandles", processAreaHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        return (IBuildResultRecordIterator) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {

                IItemQueryPage queryPage = getService().getQueryPageForBuildResultsForProcessAreas(processAreaHandles,
                        getBuildStateNames(buildStates));

                return new BuildResultRecordIterator(getRepository(), queryPage, properties);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecord getBuildResultRecord(final IBuildResultHandle buildResultHandle,
            final String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        return (IBuildResultRecord) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {

                IBuildResultRecord record = getService().getBuildResultRecord(buildResultHandle, properties);
                processBuildResultRecord(record);

                BuildRecordEventManager.getInstance().notifyRecordsReceived(new IBuildResultRecord[] { record });
                return record;
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultStatusTrend getBuildResultStatusTrend(final IBuildResultHandle buildResultHandle,
            final int maxResultCount, IProgressMonitor progressMonitor) throws TeamRepositoryException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$

        return (IBuildResultStatusTrend) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getBuildResultStatusTrend(buildResultHandle, maxResultCount);
            }
        });
    }
    
    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public IBuildResultStatusTrend[] getBuildResultStatusTrends(final IBuildResultHandle buildResultHandle,
            final int maxResultCount, IProgressMonitor progressMonitor) throws TeamRepositoryException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$

        return (IBuildResultStatusTrend[]) callCancelableService(progressMonitor, new IServiceRunnable() {
            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getBuildResultStatusTrends(buildResultHandle, maxResultCount);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecordIterator getBuildResultRecords(final String tag,
            final IProjectAreaHandle[] projectAreaHandles, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("projectAreaHandles", projectAreaHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("projectAreaHandles", projectAreaHandles); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("tag", tag); //$NON-NLS-1$
        TagsHelper.validateOnlyOneTag("tag", tag); //$NON-NLS-1$

        return (IBuildResultRecordIterator) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemQueryPage queryPage = getService().getQueryPageForBuildResultsWithTag(tag, projectAreaHandles);
                return new BuildResultRecordIterator(getRepository(), queryPage, properties);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecordIterator getBuildResultRecords(final String tag,
            final IBuildDefinitionHandle[] definitionHandles, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotEmpty("tag", tag); //$NON-NLS-1$
        TagsHelper.validateOnlyOneTag("tag", tag); //$NON-NLS-1$

        return (IBuildResultRecordIterator) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemQueryPage queryPage = getService().getQueryPageForBuildResultsWithTagForBuildDefinitions(tag,
                        definitionHandles);
                return new BuildResultRecordIterator(getRepository(), queryPage, properties);
            }
        });

    }

    @Override
    public IBuildScheduleRecord[] getBuildScheduleRecords(final IBuildDefinitionHandle buildDefinitionHandle,
            final String[] timeZoneIDs, final Timestamp dstAdjustmentDate, IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildDefinitionHandle", buildDefinitionHandle); //$NON-NLS-1$
        ValidationHelper.validateNotEmpty("timeZoneIDs", timeZoneIDs); //$NON-NLS-1$

        return (IBuildScheduleRecord[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                return getService().getBuildScheduleRecords(buildDefinitionHandle, timeZoneIDs,
                        dstAdjustmentDate);
            }
        });
    }
    
    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IItem[] save(final IBuildDefinition buildDefinition, final IBuildEngineHandle[] buildEngineHandles,
            final int[] isSupported, final IBuildEngine[] newBuildEngines, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        
        ValidationHelper.validateNotNull("buildDefinition", buildDefinition); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("buildEngineHandles", buildEngineHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNull("isSupported", isSupported); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("newBuildEngines", newBuildEngines); //$NON-NLS-1$

        class SaveBuildItemsRunnable extends ProcessRunnable {

            public IItem[] fSavedItems;

            /*
             * (non-Javadoc) Intentionally not documented. See parent.
             */
            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService(ITeamBuildService.class).saveBuildDefinition(buildDefinition,
                        buildEngineHandles, isSupported, newBuildEngines);
                fSavedItems = response.getClientItems();
                return response.getOperationReport();
            }

        }
        return (IItem[]) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                SaveBuildItemsRunnable runnable = new SaveBuildItemsRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable,
                        Messages.TeamBuildRecordClient_SAVE_BUILD_DEFINITION_PROCESS_RUNNABLE_NAME, progressMonitor);

                List savedItems = addToItemManager(runnable.fSavedItems, progressMonitor);
                return savedItems.toArray(new IItem[savedItems.size()]);
            }
        });
    }

    /**
     * Process the build result record. Process each record. This involves 3
     * steps:
     * <ul>
     * <li>Add each item contained in the record to the item manager. The item
     * manager makes a copy of the item and returns the shared item.</li>
     * <li>Assign the shared item returned from the item manager into the
     * record. This is the item that clients of the record will access.</li>
     * <li>Release the non-shared item from the record, since it isn't used
     * anymore.</li>
     * </ul>
     */
    private void processBuildResultRecord(IBuildResultRecord record) throws TeamRepositoryException {
        // Process build definition item.
        IItem sharedItem = addToItemManager(((BuildResultRecord) record).getInternalBuildDefinition());
        ((BuildResultRecordImpl) record).setBuildDefinition((IBuildDefinition) sharedItem);
        ((BuildResultRecord) record).setInternalBuildDefinition(null);

        // Process build average data item.
        sharedItem = addToItemManager(((BuildResultRecord) record).getInternalBuildAverageData());
        ((BuildResultRecordImpl) record).setBuildAverageData((IBuildAverageData) sharedItem);
        ((BuildResultRecord) record).setInternalBuildDefinition(null);

        // Process build engine item.
        sharedItem = addToItemManager(((BuildResultRecord) record).getInternalBuildEngine());
        ((BuildResultRecordImpl) record).setBuildEngine((IBuildEngine) sharedItem);
        ((BuildResultRecord) record).setInternalBuildEngine(null);

        // Process build result item.
        sharedItem = addToItemManager(((BuildResultRecord) record).getInternalBuildResult());
        ((BuildResultRecordImpl) record).setBuildResult((IBuildResult) sharedItem);
        ((BuildResultRecord) record).setInternalBuildResult(null);

        // Process requestor item.
        sharedItem = addToItemManager(((BuildResultRecord) record).getInternalRequestor());
        ((BuildResultRecordImpl) record).setRequestor((IContributor) sharedItem);
        ((BuildResultRecord) record).setInternalRequestor(null);

        // Process build request items.
        List buildRequests = ((BuildResultRecord) record).getInternalBuildRequests();
        List<IBuildRequest> sharedRequests = new ArrayList<IBuildRequest>(buildRequests.size());
        for (Object object : buildRequests) {
            sharedItem = addToItemManager((IBuildRequest) object);
            sharedRequests.add((IBuildRequest) sharedItem);
        }
        ((BuildResultRecordImpl) record).setBuildRequests(sharedRequests);
        ((BuildResultRecord) record).getInternalBuildRequests().clear();
    }

    /**
     * Process the build definition status record.
     * 
     * @see TeamBuildClient#processBuildDefinitionStatusRecord(IBuildDefinitionStatusRecord)
     */
    private void processBuildDefinitionStatusRecord(IBuildDefinitionStatusRecord record) throws TeamRepositoryException {
        IItem sharedItem = addToItemManager(((BuildDefinitionStatusRecord) record).getInternalBuildDefinition());
        ((BuildDefinitionStatusRecordImpl) record).setBuildDefinition((IBuildDefinition) sharedItem);
        ((BuildDefinitionStatusRecord) record).setInternalBuildDefinition(null);
    }

    /**
     * Process the build engine status record.
     * 
     * @see TeamBuildClient#processBuildResultRecord(IBuildResultRecord)
     */
    private void processBuildEngineStatusRecord(IBuildEngineStatusRecord record) throws TeamRepositoryException {
        IItem sharedItem = addToItemManager(((BuildEngineStatusRecord) record).getInternalBuildEngine());
        ((BuildEngineStatusRecordImpl) record).setBuildEngine((IBuildEngine) sharedItem);
        ((BuildEngineStatusRecord) record).setInternalBuildEngine(null);

        IInProgressBuild[] inProgressBuilds = record.getInProgressBuilds();
        for (IInProgressBuild build : inProgressBuilds) {
            sharedItem = addToItemManager(((InProgressBuild) build).getInternalBuildDefinition());
            ((InProgressBuildImpl) build).setBuildDefinition((IBuildDefinition) sharedItem);
            ((InProgressBuild) build).setInternalBuildDefinition(null);
        }
    }

    private String[] getBuildStateNames(BuildState[] buildStates) {
        String[] buildStateNames = null;
        if (buildStates != null && buildStates.length > 0) {
            buildStateNames = new String[buildStates.length];
            for (int i = 0; i < buildStates.length; i++) {
                buildStateNames[i] = buildStates[i].name();
            }
        }
        return buildStateNames;
    }

    private ITeamBuildService getService() {
        return (ITeamBuildService) getContext().getServiceInterface(ITeamBuildService.class);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResultRecordIterator getBuildResultRecords(final IBuildResultSearchCriteria buildQueryCriteria,
            final String[] properties, IProgressMonitor progressMonitor) throws TeamRepositoryException {
        ValidationHelper.validateNotNull("buildQueryCriteria", buildQueryCriteria); //$NON-NLS-1$

        return (IBuildResultRecordIterator) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemQueryPage queryPage = getService().getQueryPageForBuildResultsFromBuildQueryCriteria(
                        buildQueryCriteria);

                return new BuildResultRecordIterator(getRepository(), queryPage, properties);
            }
        });

    }

}
