/*******************************************************************************
 *  Licensed Materials - Property of IBM
 *  (c) Copyright IBM Corporation 2006, 2014. All Rights Reserved.
 *  
 *  Note to U.S. Government Users Restricted Rights:
 *  Use, duplication or disclosure restricted by GSA ADP Schedule
 *  Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.client.ITeamBuildRequestClient;
import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.common.TeamBuildStateException;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildEngineHandle;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.common.model.IBuildRequest;
import com.ibm.team.build.common.model.IBuildRequestHandle;
import com.ibm.team.build.common.model.IBuildRequestParams;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.common.ITeamBuildRequestService;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.process.client.IProcessClientService;
import com.ibm.team.process.common.advice.IItemsResponse;
import com.ibm.team.process.common.advice.IOperationReport;
import com.ibm.team.process.common.advice.ProcessRunnable;
import com.ibm.team.repository.client.util.IClientLibraryContext;
import com.ibm.team.repository.client.util.IClientLibraryContext.IServiceRunnable;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * The client library implementation for the team build request service.
 */
public class TeamBuildRequestClient extends AbstractTeamBuildClient implements ITeamBuildRequestClient {

    /**
     * Retrieves the build request service.
     * 
     * @return the build request service
     */
    private ITeamBuildRequestService getService() {
        return (ITeamBuildRequestService) getContext().getServiceInterface(ITeamBuildRequestService.class);
    }

    /**
     * Creates the team build request client for a specified context.
     * 
     * @param context
     *            The context to create the team build request client for.
     */
    public TeamBuildRequestClient(IClientLibraryContext context) {
        super(context);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest getNextRequest(final IBuildEngineHandle buildEngineHandle, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildEngineHandle", buildEngineHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class GetNextRequestRunnable extends ProcessRunnable {

            public IBuildRequest fBuildRequest;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().getNextRequest(buildEngineHandle, properties);
                if (response.getClientItems().length > 0) {
                    fBuildRequest = (IBuildRequest) response.getClientItems()[0];
                }
                return response.getOperationReport();
            }

        }
        return (IBuildRequest) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                GetNextRequestRunnable runnable = new GetNextRequestRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_GET_NEXT_REQUEST_OPERATION_NAME,
                        progressMonitor);

                return (IBuildRequest) addToItemManager(runnable.fBuildRequest);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest claimRequest(final IBuildRequestHandle buildRequestHandle, final IBuildEngineHandle buildEngineHandle, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildRequestHandle", buildRequestHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildEngineHandle", buildEngineHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class GetNextRequestRunnable extends ProcessRunnable {

            public IBuildRequest fBuildRequest;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().claimRequest(buildRequestHandle, buildEngineHandle, properties);
                if (response.getClientItems().length > 0) {
                    fBuildRequest = (IBuildRequest) response.getClientItems()[0];
                }
                return response.getOperationReport();
            }

        }
        return (IBuildRequest) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                GetNextRequestRunnable runnable = new GetNextRequestRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_GET_NEXT_REQUEST_OPERATION_NAME,
                        progressMonitor);

                return (IBuildRequest) addToItemManager(runnable.fBuildRequest);
            }
        });
    }
    
    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest requestBuild(IBuildDefinitionHandle buildDefinitionHandle, boolean allowDuplicateRequests,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        return requestBuild(buildDefinitionHandle, null, null, allowDuplicateRequests, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest requestBuild(IBuildDefinitionHandle buildDefinitionHandle, IBuildProperty[] buildProperties,
            boolean allowDuplicateRequests, IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException {

        return requestBuild(buildDefinitionHandle, buildProperties, null, allowDuplicateRequests, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest requestBuild(final IBuildDefinitionHandle buildDefinitionHandle,
            final IBuildProperty[] newOrModifiedBuildProperties, final IBuildProperty[] deletedBuildProperties,
            final boolean allowDuplicateRequests, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        return requestBuild(buildDefinitionHandle, newOrModifiedBuildProperties, deletedBuildProperties,
                allowDuplicateRequests, false, progressMonitor);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest requestBuild(final IBuildDefinitionHandle buildDefinitionHandle,
            final IBuildProperty[] newOrModifiedBuildProperties, final IBuildProperty[] deletedBuildProperties,
            final boolean allowDuplicateRequests, final boolean personalBuild, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildDefinitionHandle", buildDefinitionHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("newOrModifiedBuildProperties", newOrModifiedBuildProperties); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("deletedBuildProperties", deletedBuildProperties); //$NON-NLS-1$

        class RequestBuildRunnable extends ProcessRunnable {

            public IBuildRequest fBuildRequest;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().requestBuild(buildDefinitionHandle,
                        newOrModifiedBuildProperties, deletedBuildProperties, allowDuplicateRequests, personalBuild);
                fBuildRequest = (IBuildRequest) response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildRequest) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                RequestBuildRunnable runnable = new RequestBuildRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_REQUEST_BUILD_OPERATION_NAME,
                        progressMonitor);

                return (IBuildRequest) addToItemManager(runnable.fBuildRequest);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest requestBuild(final IBuildRequestParams params, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("params", params); //$NON-NLS-1$

        class RequestBuildRunnable extends ProcessRunnable {

            public IBuildRequest fBuildRequest;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().requestBuild2(params);
                fBuildRequest = (IBuildRequest) response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildRequest) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                RequestBuildRunnable runnable = new RequestBuildRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_REQUEST_BUILD_OPERATION_NAME,
                        progressMonitor);

                return (IBuildRequest) addToItemManager(runnable.fBuildRequest);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest requestAndStartBuild(final IBuildDefinitionHandle buildDefinitionHandle,
            final IBuildEngineHandle buildEngineHandle, final IProgressMonitor progressMonitor)
            throws TeamRepositoryException, TeamBuildException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildDefinitionHandle", buildDefinitionHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildEngineHandle", buildEngineHandle); //$NON-NLS-1$

        class RequestAndStartBuildRunnable extends ProcessRunnable {

            public IBuildRequest fBuildRequest;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().requestAndStartBuild(buildDefinitionHandle, buildEngineHandle);
                fBuildRequest = (IBuildRequest) response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildRequest) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                RequestAndStartBuildRunnable runnable = new RequestAndStartBuildRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_REQUEST_BUILD_OPERATION_NAME,
                        progressMonitor);

                return (IBuildRequest) addToItemManager(runnable.fBuildRequest);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest cancelPendingRequest(final IBuildRequestHandle buildRequestHandle, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        ValidationHelper.validateNotNull("buildRequestHandle", buildRequestHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class CancelPendingRequestRunnable extends ProcessRunnable {

            public IBuildRequest fCanceledRequest;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().cancelPendingRequest(buildRequestHandle, properties);
                fCanceledRequest = (IBuildRequest) response.getClientItems()[0];
                return response.getOperationReport();
            }

        }
        return (IBuildRequest) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                CancelPendingRequestRunnable runnable = new CancelPendingRequestRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_CANCEL_BUILD_REQUEST_OPERATION_NAME,
                        progressMonitor);

                return (IBuildRequest) addToItemManager(runnable.fCanceledRequest);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResult makeBuildComplete(final IBuildResultHandle buildResultHandle, final boolean buildNotNecessary,
            final String[] properties, final IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class CompleteBuildRunnable extends ProcessRunnable {

            public IBuildResult fCompleteBuild;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().makeBuildComplete(buildResultHandle, buildNotNecessary,
                        properties);
                if (response.getClientItems().length > 0) {
                    fCompleteBuild = (IBuildResult) response.getClientItems()[0];
                }
                return response.getOperationReport();
            }

        }
        return (IBuildResult) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                CompleteBuildRunnable runnable = new CompleteBuildRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_COMPLETE_BUILD_OPERATION_NAME,
                        progressMonitor);

                IBuildResult completedBuild = runnable.fCompleteBuild;
                if (completedBuild == null) {
                    deleteFromItemManager(buildResultHandle);
                } else {
                    completedBuild = (IBuildResult) addToItemManager(completedBuild);
                }
                return completedBuild;
            }
        });
    }

    @Override
    public IBuildResult makeBuildCompleteWithStatus(final IBuildResultHandle buildResultHandle,
            final BuildStatus buildStatus,
            final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException,
            TeamBuildStateException, IllegalArgumentException {
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildStatus", buildStatus); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class CompleteBuildRunnable extends ProcessRunnable {

            public IBuildResult fCompleteBuild;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().makeBuildCompleteWithStatus(buildResultHandle, buildStatus.name(),
                        properties);
                fCompleteBuild = (IBuildResult) response.getClientItems()[0];
                return response.getOperationReport();
            }

        }
        return (IBuildResult) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                CompleteBuildRunnable runnable = new CompleteBuildRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_COMPLETE_BUILD_OPERATION_NAME,
                        progressMonitor);

                IBuildResult completedBuild = runnable.fCompleteBuild;
                return (IBuildResult) addToItemManager(completedBuild);
            }
        });
    }

    @Override
    public IBuildResult makeBuildIncomplete(final IBuildResultHandle buildResultHandle, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class AbandonBuildRunnable extends ProcessRunnable {

            public IBuildResult fIncompleteBuild;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().makeBuildIncomplete(buildResultHandle, properties);
                fIncompleteBuild = (IBuildResult) response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildResult) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                AbandonBuildRunnable runnable = new AbandonBuildRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_ABANDON_BUILD_OPERATION_NAME,
                        progressMonitor);

                return (IBuildResult) addToItemManager(runnable.fIncompleteBuild);
            }
        });

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildRequest getNextRequest(final String buildDefinitionId, final IBuildEngineHandle buildEngineHandle,
            final String[] properties, final IProgressMonitor progressMonitor) throws TeamRepositoryException,
            IllegalArgumentException {
        
        ValidationHelper.validateNotEmpty("buildDefinitionId", buildDefinitionId); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildEngineHandle", buildEngineHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class GetNextRequestRunnable extends ProcessRunnable {

            public IBuildRequest fBuildRequest;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().getNextRequestByBuildDefinition(buildDefinitionId,
                        buildEngineHandle, properties);
                if (response.getClientItems().length > 0) {
                    fBuildRequest = (IBuildRequest) response.getClientItems()[0];
                }
                return response.getOperationReport();
            }

        }
        return (IBuildRequest) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                GetNextRequestRunnable runnable = new GetNextRequestRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_GET_NEXT_REQUEST_OPERATION_NAME,
                        progressMonitor);

                return (IBuildRequest) addToItemManager(runnable.fBuildRequest);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public IBuildResult startBuild(final IBuildRequestHandle buildRequestHandle, final String[] properties,
            final IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {
        
        ValidationHelper.validateNotNull("buildRequestHandle", buildRequestHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        class StartBuildRunnable extends ProcessRunnable {

            public IBuildResult fStartedBuild;

            @Override
            public IOperationReport run(IProgressMonitor monitor) throws TeamRepositoryException {
                IItemsResponse response = getService().startBuild(buildRequestHandle, properties);
                fStartedBuild = (IBuildResult) response.getFirstClientItem();
                return response.getOperationReport();
            }

        }
        return (IBuildResult) callCancelableService(progressMonitor, new IServiceRunnable() {

            public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
                StartBuildRunnable runnable = new StartBuildRunnable();
                IProcessClientService processService = (IProcessClientService) getRepository().getClientLibrary(
                        IProcessClientService.class);
                processService.execute(runnable, Messages.TeamBuildRequestClient_START_BUILD_OPERATION_NAME,
                        progressMonitor);

                return (IBuildResult) addToItemManager(runnable.fStartedBuild);
            }
        });

    }
}