/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client.iterator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.client.iterator.IBuildResultRecordIterator;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.internal.client.ITeamBuildRecordClient;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.query.IItemQueryPage;

/**
 * Implementation of {@link IBuildResultRecordIterator}.
 */
public class BuildResultRecordIterator implements IBuildResultRecordIterator {

    
    /**
     * Constructs an iterator that will iterate through all build records in the
     * repository.
     * 
     * @param teamRepository
     *            The repository containing the build result records to iterate
     *            through.
     * @return An empty iterator.
     */
    public static IBuildResultRecordIterator getEmptyResultRecordIterator(ITeamRepository teamRepository) {
        return new BuildResultRecordIterator(teamRepository);
    }

    private IItemQueryPage fQueryPage;
    private Iterator fQueryIterator;
    private final ITeamBuildRecordClient fRecordClient;
    private final String[] fProperties;
    protected int fPageSize = MAX_BUILD_RESULT_RECORDS;

    /**
     * Constructs an iterator that will iterate through all build records in the
     * repository.
     * 
     * @param teamRepository
     *            The repository containing the build result records to iterate
     *            through.
     * @param queryPage
     *            The query page of build result item handles to begin iteration
     *            at.
     * @param properties
     *            The properties that should be included in the build result
     *            contained in the build result record.
     */
    public BuildResultRecordIterator(ITeamRepository teamRepository, IItemQueryPage queryPage, String[] properties) {
        ValidationHelper.validateNotNull("teamRepository", teamRepository); //$NON-NLS-1$
        ValidationHelper.validateNotNull("queryPage", queryPage); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("properties", properties); //$NON-NLS-1$

        fRecordClient = (ITeamBuildRecordClient) teamRepository.getClientLibrary(ITeamBuildRecordClient.class);
        fProperties = properties;
        fQueryPage = queryPage;
        fQueryIterator = queryPage.getItemHandles().iterator();

    }
    
    private BuildResultRecordIterator(ITeamRepository teamRepository) {
        fRecordClient = (ITeamBuildRecordClient) teamRepository.getClientLibrary(ITeamBuildRecordClient.class);
        fProperties = new String[0];
        fQueryPage = IItemQueryPage.EMPTY_RESULT;
        fQueryIterator = fQueryPage.getItemHandles().iterator();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public synchronized boolean hasNext() throws TeamRepositoryException {
        updatePosition(null);
        return fQueryIterator.hasNext();

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public synchronized IBuildResultRecord next(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        if (progressMonitor == null) {
            progressMonitor = new NullProgressMonitor();
        }
        if (!progressMonitor.isCanceled()) {
            updatePosition(progressMonitor);
            if (fQueryIterator.hasNext()) {
                return getBuildResultRecord((IItemHandle) fQueryIterator.next(), progressMonitor);
            } else {
                return null;
            }
        } else {
            throw new OperationCanceledException();
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public synchronized IBuildResultRecord[] next(int numRecords, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        if (numRecords > MAX_BUILD_RESULT_RECORDS) {
            throw new IllegalArgumentException(NLS.bind(
                    Messages.BuildResultRecordIterator_MAX_RECORD_ITERATION_EXCEEDED, MAX_BUILD_RESULT_RECORDS));
        }
        if (progressMonitor == null) {
            progressMonitor = new NullProgressMonitor();
        }
        ArrayList<IItemHandle> itemHandles = new ArrayList<IItemHandle>();
        for (int i = 0; i < numRecords && !progressMonitor.isCanceled(); i++) {
            updatePosition(progressMonitor);
            if (fQueryIterator.hasNext()) {
                itemHandles.add((IItemHandle) fQueryIterator.next());
            } else {
                break;
            }
        }
        if (progressMonitor.isCanceled()) {
            throw new OperationCanceledException();
        }
        return getBuildResultRecords(itemHandles, progressMonitor);

    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public int size() {
        return fQueryPage == null ? 0 : fQueryPage.getResultSize();
    }

    private void updatePosition(IProgressMonitor progressMonitor) throws TeamRepositoryException {

        while (!fQueryIterator.hasNext() && fQueryPage.hasNext()) {

            fQueryPage = (IItemQueryPage) fRecordClient.fetchPage(fQueryPage.getToken(),
                    fQueryPage.getNextStartPosition(), fPageSize, progressMonitor);

            fQueryIterator = fQueryPage.getItemHandles().iterator();
        }

    }

    private IBuildResultRecord[] getBuildResultRecords(ArrayList<IItemHandle> itemHandles,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {
        IBuildResultHandle[] resultHandles = new IBuildResultHandle[itemHandles.size()];
        for (int i = 0; i < resultHandles.length; i++) {
            resultHandles[i] = (IBuildResultHandle) IBuildResult.ITEM_TYPE.createItemHandle(
                    itemHandles.get(i).getItemId(), null);
        }
        IBuildResultRecord[] records = fRecordClient.getBuildResultRecords(resultHandles, fProperties, progressMonitor);
        LinkedList<IBuildResultRecord> nonNullRecords = new LinkedList<IBuildResultRecord>();
        for (IBuildResultRecord buildResultRecord : records) {
            if (buildResultRecord != null) {
                nonNullRecords.add(buildResultRecord);
            }
        }
        if (records.length == nonNullRecords.size()) {
            return records;
        } else {
            return nonNullRecords.toArray(new IBuildResultRecord[nonNullRecords.size()]);
        }
    }

    private IBuildResultRecord getBuildResultRecord(IItemHandle itemHandle, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        IBuildResultHandle buildResultHandle = (IBuildResultHandle) IBuildResult.ITEM_TYPE.createItemHandle(
                itemHandle.getItemId(), null);
        return fRecordClient.getBuildResultRecord(buildResultHandle, fProperties, progressMonitor);
    }
}
