/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client.iterator;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.common.iterator.AbstractItemQueryIterator;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.query.IItemQuery;
import com.ibm.team.repository.common.query.IItemQueryPage;
import com.ibm.team.repository.common.query.IQuery;
import com.ibm.team.repository.common.service.IQueryService;

/**
 * Provides the ability to iterate through the item handles of an {@link IQuery}
 * on the client.
 * 
 * @param <T>
 *            The type of item handle that the iterator will return.
 */
public final class ItemQueryIterator<T extends IItemHandle> extends AbstractItemQueryIterator<T> {

    /**
     * The maximum number of item handles per page.
     */
    public static int MAX_PAGE_SIZE = IQueryService.ITEM_QUERY_MAX_PAGE_SIZE;
    
    private ITeamBuildClient fBuildClient;
    private IItemQueryPage fQueryPage;
    private int fQueryPageSize;

    /**
     * Constructs an empty query item iterator.
     */
    public ItemQueryIterator() {
        super();

        fQueryPage = IItemQueryPage.EMPTY_RESULT;
    }

    /**
     * Constructs the query item iterator.
     * 
     * @param buildClient
     *            the build client used to execute the query.
     * @param query
     *            the query to iterate through
     * @param queryParams
     *            the parameters to the query.
     * @throws IllegalArgumentException
     *             if any of the parameters to the constructor are
     *             <code>null</code>.
     */
    public ItemQueryIterator(ITeamBuildClient buildClient, IItemQuery query, Object[] queryParams)
            throws IllegalArgumentException {
        this(buildClient, query, queryParams, MAX_PAGE_SIZE);
    }

    /**
     * Constructs the query item iterator.
     * 
     * @param buildClient
     *            the build client used to execute the query.
     * @param query
     *            the query to iterate through
     * @param queryParams
     *            the parameters to the query.
     * @param queryPageSize
     *            the size of each page of handles in the query.
     * @throws IllegalArgumentException
     *             if any of the parameters to the constructor are
     *             <code>null</code>.
     */
    public ItemQueryIterator(ITeamBuildClient buildClient, IItemQuery query, Object[] queryParams, int queryPageSize)
            throws IllegalArgumentException {
        super(query, queryParams);

        ValidationHelper.validateNotNull("buildClient", buildClient); //$NON-NLS-1$

        fQueryPageSize = queryPageSize;
        fBuildClient = buildClient;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public int size(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        updatePosition(progressMonitor);
        return fQueryPage.getResultSize();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected synchronized void updatePosition(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        if (!hasQueryExecuted()) {
            if (isCommitted()) {
                fQueryPage = queryItems(fQuery, fQueryParams, fQueryPageSize, progressMonitor);
            } else {
                fQueryPage = uncommittedQueryItems(fQuery, fQueryParams, fQueryPageSize, progressMonitor);
            }
            fHandleIterator = fQueryPage.getItemHandles().iterator();

        }
        while (!fHandleIterator.hasNext() && fQueryPage.hasNext()) {
            fQueryPage = fetchPage(fQueryPage, fQueryPageSize, progressMonitor);
            fHandleIterator = fQueryPage.getItemHandles().iterator();
        }
    }

    /**
     * Fetches the next page of the query.
     * 
     * @param queryPage
     *            the current page of the query.
     * @param queryPageSize
     *            the size of the page of the query.
     * @param progressMonitor
     *            the progress monitor to track progress on.
     * @return The next page for the executed query.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    private IItemQueryPage fetchPage(IItemQueryPage queryPage, int queryPageSize, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {
        return (IItemQueryPage) fBuildClient.fetchPage(queryPage.getToken(), queryPage.getNextStartPosition(),
                queryPageSize, progressMonitor);
    }

    /**
     * Executes the committed query.
     * 
     * @param itemQuery
     *            the query to execute
     * @param queryParams
     *            the parameters to supply to the query.
     * @param queryPageSize
     *            the size of the pages of the query.
     * @param progressMonitor
     *            the progress monitor to track progress on.
     * @return The first page for the executed query.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    private IItemQueryPage queryItems(IItemQuery itemQuery, Object[] queryParams, int queryPageSize,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {
        return fBuildClient.queryItems(itemQuery, queryParams, queryPageSize, progressMonitor);
    }

    /**
     * Executes the uncommitted query.
     * 
     * @param itemQuery
     *            the query to execute
     * @param queryParams
     *            the parameters to supply to the query.
     * @param queryPageSize
     *            the size of the pages of the query.
     * @param progressMonitor
     *            the progress monitor to track progress on.
     * @return The first page for the executed query.
     * @throws TeamRepositoryException
     *             If an error occurs while accessing the repository.
     */
    private IItemQueryPage uncommittedQueryItems(IItemQuery itemQuery, Object[] queryParams, int queryPageSize,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {
        return fBuildClient.uncommittedQueryItems(itemQuery, queryParams, queryPageSize, progressMonitor);
    }
}
