/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Locale;
import java.util.StringTokenizer;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;

import com.ibm.team.build.common.TeamBuildException;
import com.ibm.team.build.internal.common.helper.FileHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.repository.client.IContentManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Utility class to help manage conversion to and from <code>IContent</code>
 */
public class ContentUtil {

    static {

        // Instantiate an instance to get around code coverage limitation that
        // it can't currently exclude intentionally private constructors
        new ContentUtil();
    }

    /**
     * Private default constructor to prevent instances.
     */
    private ContentUtil() {
        // Don't want any instances
    }

    /**
     * Attempt to use the java mime-mappings to determine the content type of
     * the specified file.
     * 
     * @param fileName
     *            The file name for which to determine the content type.
     * @return the content type or <code>null</code> if it could not be determined.
     */
    public static String getContentTypeFromFileName(String fileName) {
        ValidationHelper.validateNotEmpty("fileName", fileName); //$NON-NLS-1$

        String contentType = URLConnection.getFileNameMap().getContentTypeFor(fileName);
        if (contentType == null && FileHelper.hasFileSuffix(fileName, ".log")) { //$NON-NLS-1$
            contentType = IContent.CONTENT_TYPE_TEXT;
        }
        return contentType;
    }

    /**
     * Return true if the specified content type is a text mime type, false
     * otherwise.
     * 
     * @param contentType
     *            The content type to examine.
     * @return true if the specified content type is a text mime type, false
     *         otherwise.
     */
    public static boolean isTextContentType(String contentType) {
        if (contentType != null && contentType.trim().toLowerCase(Locale.ENGLISH).startsWith("text")) { //$NON-NLS-1$
            return true;
        }

        return false;
    }

    /**
     * Converts a <code>String</code> into content stored in the repository.
     * 
     * @param teamRepository
     *            The repository to store the content in.
     * @param stringContent
     *            The <code>String</code> to convert to content.
     * @return The content the <code>String</code> was converted to or
     *         <code>null</code> if the specified <oode>stringContent</code>
     *         or <code>teamRepository</code> was <code>null</code>.
     * @throws TeamRepositoryException
     *             if the content could not be stored.
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IContent stringToContent(ITeamRepository teamRepository, String stringContent)
            throws TeamRepositoryException {
        return stringToContent(teamRepository, stringContent, null);
    }

    /**
     * Converts a <code>String</code> into content stored in the repository.
     * 
     * @param teamRepository
     *            The repository to store the content in.
     * @param stringContent
     *            The <code>String</code> to convert to content.
     * @param progressMonitor
     *            Used to track progress of storing the content. This can be
     *            <code>null</code>.
     * @return The content the <code>String</code> was converted to or
     *         <code>null</code> if the specified <oode>stringContent</code>
     *         or <code>teamRepository</code> was <code>null</code>.
     * @throws TeamRepositoryException
     *             if the content could not be stored.
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IContent stringToContent(ITeamRepository teamRepository, String stringContent,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {
        IContent content = null;
        if (null != teamRepository && null != stringContent) {
            content = teamRepository.contentManager().storeContent(IContent.CONTENT_TYPE_TEXT, stringContent,
                    progressMonitor);
        }
        return content;
    }

    /**
     * Converts the contents of an <code>IContent</code> instance into a
     * <code>String</code>.
     * 
     * @param teamRepository
     *            The repository to retrieve the content from.
     * @param content
     *            The content instance to convert to a string.
     * @param progressMonitor
     *            Used to track the progress of the retrieval of the content on.
     *            This can be <code>null</code>.
     * @return The contents of the specified <code>IContent</code> instance.
     *         Returns <code>null</code> if the specified <code>content</code>
     *         or <code>teamRepository</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If the content could not be retrieved from the repository.
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static String contentToString(ITeamRepository teamRepository, IContent content,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {

        String stringContent = ""; //$NON-NLS-1$
        if (null != content && null != teamRepository) {
            final int BUFFER_SIZE = 8192;
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream(BUFFER_SIZE);
            IContentManager contentManager = teamRepository.contentManager();
            contentManager.retrieveContent(content, outputStream, progressMonitor);
            String encoding = content.getCharacterEncoding();
            if (encoding == null) {
                encoding = IContent.ENCODING_UTF_8;
            }
            try {
                stringContent = outputStream.toString(encoding);
            } catch (UnsupportedEncodingException e) {
                throw new TeamBuildException(e);
            }
        }
        return stringContent;
    }

    /**
     * Converts the contents of an <code>IContent</code> instance into a
     * <code>File</code>.
     * 
     * @param teamRepository
     *            The repository to retrieve the content from.
     * @param content
     *            The content instance to convert to a string.
     * @param monitor
     *            Used to track the progress of the retrieval of the content on.
     *            This can be <code>null</code>.
     * @param filePath
     *            The full path of the File that the <code>IContent</code>
     *            instance will be read into. File will be created if it does
     *            not exist.
     * @return A <code>File</code>, with the given filePath containing the
     *         contents of the specified <code>IContent</code> instance.
     *         Returns <code>null</code> if the specified <code>content</code>
     *         or <code>teamRepository</code> is <code>null</code>.
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     * @throws TeamRepositoryException
     *             If the content could not be retrieved from the repository
     */
    public static File contentToFile(ITeamRepository teamRepository, IContent content, IProgressMonitor monitor,
            String filePath) throws TeamRepositoryException {

        if (monitor == null) {
            monitor = new NullProgressMonitor();
        }

        File outFile = null;

        if (null != content && null != teamRepository) {
            FileOutputStream outputStream = null;
            InputStream inputStream = null;

            try {
                outFile = new File(filePath);
                if (!outFile.exists()) {
                    outFile.createNewFile();
                }
                outputStream = new FileOutputStream(outFile);

                IContentManager contentManager = teamRepository.contentManager();
                inputStream = contentManager.retrieveContentStream(content, monitor);
                copyFile(inputStream, outputStream, (int) content.getEstimatedConvertedLength(), monitor);
            } catch (IOException ioe) {
                TeamRepositoryException tre = new TeamRepositoryException(ioe);
                tre.setOrigin(teamRepository);
                throw tre;
            } finally {
                if (outputStream != null) {
                    try {
                        outputStream.close();
                    } catch (IOException exception) {
                        // Ignore.
                    }
                }

                if (inputStream != null) {
                    try {
                        inputStream.close();
                    } catch (IOException exception) {
                        // Ignore.
                    }
                }
            }
        }
        return outFile;
    }

    private static void copyFile(InputStream inStream, OutputStream outStream, int length, IProgressMonitor monitor)
            throws IOException {
        byte[] buffer = new byte[32000];

        monitor.beginTask(Messages.ContentUtil_RETRIEVING_CONTENT_PROGRESS, length);

        try {
            for (int numbytes = inStream.read(buffer); numbytes > -1; numbytes = inStream.read(buffer)) {
                if (monitor.isCanceled()) {
                    throw new OperationCanceledException();
                }

                outStream.write(buffer, 0, numbytes);
//                monitor.worked(numbytes);
            }
        } finally {
            monitor.done();
        }
    }

    /**
     * Converts an array of <code>String</code> into content stored in the
     * repository.
     * 
     * @param teamRepository
     *            The repository to store the content in.
     * @param strings
     *            The <code>String</code>s to convert to content.
     * @param delimiter
     *            The delimeter character to use when persisting the strings. If
     *            any string in the given array contains the delimiter
     *            character, the strings cannot be reliably retrieved.
     * @return The content the <code>String</code>s were converted to.
     *         Returns null in any of these cases:
     *         <ul>
     *         <li>the given teamRepository is null</li>
     *         <li>the given string array is null</li>
     *         <li>the given string array is length 0</li>
     *         <li>the given string array contains any null strings</li>
     *         <li>the given string array contains any empty strings</li>
     *         </ul>
     * @throws TeamRepositoryException
     *             if the content could not be stored.
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IContent stringArrayToContent(ITeamRepository teamRepository, String[] strings, char delimiter)
            throws TeamRepositoryException {

        if (teamRepository == null || strings == null || strings.length == 0) {
            return null;
        }

        IContent content = null;

        if (null != teamRepository && null != strings && strings.length > 0) {
            StringBuffer stringContent = new StringBuffer();
            for (int i = 0; i < strings.length; i++) {
                if (strings[i] == null || strings[i].equals("")) { //$NON-NLS-1$
                    return null;
                }

                stringContent.append(strings[i]);
                // Don't want a trailing delimiter.
                if (i < strings.length - 1) {
                    stringContent.append(delimiter);
                }
            }

            content = teamRepository.contentManager().storeContent(IContent.CONTENT_TYPE_TEXT,
                    stringContent.toString(), null);
        }
        return content;
    }

    /**
     * Converts the contents of an <code>IContent</code> instance into an
     * array of <code>String</code>.
     * 
     * @param teamRepository
     *            The repository to retrieve the content from.
     * @param content
     *            The content instance to convert to a string.
     * @param delimiter
     *            The delimiter character used in the call to
     *            <code>stringArrayToContent</code>.
     * @param progressMonitor
     *            to monitor the progress of this method.
     * @return The contents of the specified <code>IContent</code> instance.
     *         Returns an empty array if the content is empty. Returns
     *         <code>null</code> if the specified <code>content</code> or
     *         <code>teamRepository</code> is <code>null</code>.
     * @throws TeamRepositoryException
     *             If the content could not be retrieved from the repository.
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static String[] contentToStringArray(ITeamRepository teamRepository, IContent content, char delimiter,
            IProgressMonitor progressMonitor) throws TeamRepositoryException {

        if (teamRepository == null || content == null) {
            return null;
        }

        ArrayList strings = new ArrayList();
        if (null != content && null != teamRepository) {
            final int BUFFER_SIZE = 8192;
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream(BUFFER_SIZE);
            IContentManager contentManager = teamRepository.contentManager();
            contentManager.retrieveContent(content, outputStream, progressMonitor);
            String stringContent = outputStream.toString();
            if (stringContent.length() > 0) {
                String delimiterString = new String(new char[] { delimiter });
                StringTokenizer tokenizer = new StringTokenizer(stringContent, delimiterString);
                while (tokenizer.hasMoreTokens()) {
                    strings.add(tokenizer.nextToken());
                }
            }
        }
        return (String[]) strings.toArray(new String[strings.size()]);
    }

}
