/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.client.workitem;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.WorkItemConstants;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.common.model.IBuildResult;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.IBuildResultHandle;
import com.ibm.team.build.common.model.query.IBaseBuildResultQueryModel.IBuildResultQueryModel;
import com.ibm.team.build.internal.client.ITeamBuildRecordClient;
import com.ibm.team.build.internal.client.iterator.ItemQueryIterator;
import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.common.helper.ItemHelper;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.common.links.BuildLinkTypes;
import com.ibm.team.build.internal.common.model.dto.IBuildResultRecord;
import com.ibm.team.links.client.ILinkManager;
import com.ibm.team.links.common.ILink;
import com.ibm.team.links.common.ILinkCollection;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.internal.links.LinksPackage;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;
import com.ibm.team.repository.common.IHelperType;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.repository.common.query.IItemQuery;
import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.common.workflow.IWorkflowInfo;

/**
 * Helper methods for linking work items with a build result.
 */
public class WorkItemHelper {

    static {

        // Instantiate an instance to get around code coverage limitation that
        // it can't currently exclude intentionally private constructors
        new WorkItemHelper();
    }

    /**
     * Private default constructor to prevent instances.
     */
    private WorkItemHelper() {
        // Don't want any instances
    }

    /**
     * The delimiter to use for the build result extended contribution data for
     * fixed work items.
     */
    private static final char EXTENDED_DATA_DELIMITER = '\n';

    /**
     * Sets the list of work items as the "fixed in build" work items for the
     * given build, overwriting any previous fixed in build work items. This
     * method causes the given build result to be updated and saved.
     * 
     * @param teamRepository
     *            the repository where the build result and work items are found
     * @param buildResultHandle
     *            the build result to update
     * @param workItemHandles
     *            the fixed work items
     * @return the updated build result
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null or empty
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IBuildResult setFixedInBuild(ITeamRepository teamRepository, IBuildResultHandle buildResultHandle,
            IWorkItemHandle[] workItemHandles) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("teamRepository", teamRepository); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("workItemHandles", workItemHandles); //$NON-NLS-1$
        ValidationHelper.validateNotNullElements("workItemHandles", workItemHandles); //$NON-NLS-1$

        String[] handleIds = new String[workItemHandles.length];
        for (int i = 0; i < workItemHandles.length; i++) {
            handleIds[i] = workItemHandles[i].getItemId().getUuidValue();
        }

        IBuildResultContribution contribution = BuildItemFactory.createBuildResultContribution();
        contribution.setExtendedContributionTypeId(WorkItemConstants.EXTENDED_DATA_TYPE_ID);
        contribution.setExtendedContributionData(ContentUtil.stringArrayToContent(teamRepository, handleIds,
                EXTENDED_DATA_DELIMITER));
        return ClientFactory.getTeamBuildClient(teamRepository).addBuildResultContribution(buildResultHandle,
                contribution, IBuildResult.PROPERTIES_COMPLETE, null);
    }

    /**
     * Gets the work items fixed in the given build result.
     * 
     * @param teamRepository
     *            The repository where the build result is found.
     * @param buildResultHandle
     *            The build result with the fixed work items.
     * @param monitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The fixed work items or an empty array if none were fixed in the
     *         build; never null.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IWorkItemHandle[] getFixedInBuild(ITeamRepository teamRepository,
            IBuildResultHandle buildResultHandle, IProgressMonitor monitor) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("teamRepository", teamRepository); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$

        List workItemHandles = new ArrayList();

        SubMonitor subMonitor = SubMonitor.convert(monitor, 2);
        IBuildResultContribution[] buildResultContributions = ClientFactory.getTeamBuildClient(teamRepository).getBuildResultContributions(
                buildResultHandle, WorkItemConstants.EXTENDED_DATA_TYPE_ID, subMonitor.newChild(1));

        subMonitor.setWorkRemaining(buildResultContributions.length);
        for (IBuildResultContribution contribution : buildResultContributions) {
            IContent content = contribution.getExtendedContributionData();
            if (content != null) {
                String[] workItemHandleIds = ContentUtil.contentToStringArray(teamRepository, content,
                        EXTENDED_DATA_DELIMITER, subMonitor.newChild(1));
                for (int i = 0; i < workItemHandleIds.length; i++) {
                    workItemHandles.add(IWorkItem.ITEM_TYPE.createItemHandle(UUID.valueOf(workItemHandleIds[i]), null));
                }
            }
        }
        return (IWorkItemHandle[]) workItemHandles.toArray(new IWorkItemHandle[workItemHandles.size()]);
    }
    
    /**
     * Get the work items included in the build result, if any.
     * 
     * @param teamRepository
     *            the repository where the build result is found
     * @param buildResultHandle
     *            the build result to check
     * @param monitor
     *            A progress monitor or <code>null</code> if progress
     *            monitoring is not desired.
     * @return The work items reported against the build result. The array may
     *         be empty, but is never <code>null</code>.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IWorkItemHandle[] getIncludedWorkItems(ITeamRepository teamRepository,
            IBuildResultHandle buildResultHandle, IProgressMonitor monitor) throws TeamRepositoryException {
        String buildLinkType = BuildLinkTypes.INCLUDED_WORK_ITEMS;

        return getLinkedWorkItems(teamRepository, buildResultHandle, buildLinkType, monitor);
    }

    /**
     * Get the work items reported against the build result, if any.
     * 
     * @param teamRepository
     *            the repository where the build result is found
     * @param buildResultHandle
     *            the build result to check
     * @param monitor
     *            A progress monitor or <code>null</code> if progress
     *            monitoring is not desired.
     * @return The work items reported against the build result. The array may
     *         be empty, but is never <code>null</code>.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IWorkItemHandle[] getReportedWorkItems(ITeamRepository teamRepository,
            IBuildResultHandle buildResultHandle, IProgressMonitor monitor) throws TeamRepositoryException {
        String buildLinkType = BuildLinkTypes.REPORTED_WORK_ITEMS;

        return getLinkedWorkItems(teamRepository, buildResultHandle, buildLinkType, monitor);
    }

    /**
     * Get the work items linked to the build result, if any.
     * 
     * @param teamRepository
     *            the repository where the build result is found
     * @param buildResultHandle
     *            the build result to check
     * @param buildLinkType
     *            the type of link to search for
     * @param monitor
     *            A progress monitor or <code>null</code> if progress monitoring
     *            is not desired.
     * @return The work items reported against the build result. The array may
     *         be empty, but is never <code>null</code>.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IWorkItemHandle[] getLinkedWorkItems(ITeamRepository teamRepository,
            IBuildResultHandle buildResultHandle, String buildLinkType, IProgressMonitor monitor)
            throws TeamRepositoryException {
        ValidationHelper.validateNotNull("teamRepository", teamRepository); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildLinkType", buildLinkType); //$NON-NLS-1$

        List<IWorkItemHandle> reportedWorkItems = new ArrayList<IWorkItemHandle>();

        ILinkManager linkManager = (ILinkManager) teamRepository.getClientLibrary(ILinkManager.class);
        IReference source = linkManager.referenceFactory().createReferenceToItem(buildResultHandle);
        ILinkCollection referencesToResults = linkManager.findLinksBySource(buildLinkType, source,
                monitor).getAllLinksFromHereOn();

        for (Object object : referencesToResults) {
            ILink link = (ILink) object;
            IWorkItemHandle workItemHandle = (IWorkItemHandle) link.getTargetRef().resolve();
            if (workItemHandle != null) {
                reportedWorkItems.add(workItemHandle);
            }
        }

        return (IWorkItemHandle[]) reportedWorkItems.toArray(new IWorkItemHandle[reportedWorkItems.size()]);
    }

    /**
     * Get the unresolved work items that are linked to any builds of the given
     * build definition.
     * 
     * @param teamRepository
     *            the repository where the build result is found
     * @param buildDefinition
     *            the build definition
     * @param monitor
     *            A progress monitor or <code>null</code> if progress
     *            monitoring is not desired.
     * @return The unresolved work items. The array may be empty, but is never
     *         <code>null</code>.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static IWorkItemHandle[] getUnresolvedWorkItems(ITeamRepository teamRepository,
            IBuildDefinitionHandle buildDefinition, IProgressMonitor monitor) throws TeamRepositoryException {

        ValidationHelper.validateNotNull("teamRepository", teamRepository); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildDefinition", buildDefinition); //$NON-NLS-1$

        ITeamBuildClient client = (ITeamBuildClient) teamRepository.getClientLibrary(ITeamBuildClient.class);
        IBuildResultQueryModel queryModel = IBuildResultQueryModel.ROOT;
        IItemQuery query = IItemQuery.FACTORY.newInstance(queryModel);
        query.filter(queryModel.buildDefinition()._eq(query.newItemHandleArg()));
        ItemQueryIterator<IBuildResultHandle> iter = new ItemQueryIterator<IBuildResultHandle>(client, query,
                new Object[] { buildDefinition });

        ILinkManager linkManager = (ILinkManager) teamRepository.getClientLibrary(ILinkManager.class);

        IReference[] sources = new IReference[iter.size(monitor)];
        for (int i = 0; i < iter.size(monitor); i++) {
            sources[i] = linkManager.referenceFactory().createReferenceToItem(iter.next(monitor));
        }

        ILinkCollection referencesToResults = linkManager.findLinksBySource(
                new String[] { BuildLinkTypes.REPORTED_WORK_ITEMS }, sources, monitor).getAllLinksFromHereOn();

        List<IWorkItemHandle> reportedWorkItems = new ArrayList<IWorkItemHandle>();
        for (Object object : referencesToResults) {
            ILink link = (ILink) object;
            IWorkItemHandle workItemHandle = (IWorkItemHandle) link.getTargetRef().resolve();
            if (workItemHandle != null) {
                reportedWorkItems.add(workItemHandle);
            }
        }

        List properties = new ArrayList();
        properties.add(IWorkItem.TYPE_PROPERTY);
        properties.add(IWorkItem.STATE_PROPERTY);
        properties.add(IWorkItem.WORKFLOW_SURROGATE_PROPERTY);
        properties.add(IWorkItem.PROJECT_AREA_PROPERTY);
        List workItems = teamRepository.itemManager().fetchPartialItems(reportedWorkItems, IItemManager.DEFAULT,
                properties, monitor);

        Map<String, IWorkItemHandle> unresolvedWorkItems = new HashMap();

        for (Object object : workItems) {
            if (object != null) {
                IWorkItem workItem = (IWorkItem) object;
                if (!isResolved(workItem, teamRepository, monitor)) {
                    unresolvedWorkItems.put(workItem.getItemId().getUuidValue(), workItem);
                }
            }
        }

        return (IWorkItemHandle[]) unresolvedWorkItems.values().toArray(new IWorkItemHandle[unresolvedWorkItems.size()]);
    }

    private static boolean isResolved(IWorkItem workItem, ITeamRepository repository, IProgressMonitor monitor)
            throws TeamRepositoryException {
        IWorkItemClient client = (IWorkItemClient) repository.getClientLibrary(IWorkItemClient.class);
        IWorkflowInfo workflowInfo = client.findWorkflowInfo(workItem, monitor);
        return workflowInfo != null && workflowInfo.getStateGroup(workItem.getState2()) == IWorkflowInfo.CLOSED_STATES;
    }

    /**
     * Adds a reported work item link to the build result. This method creates a
     * link between the build result and work item. Neither object is modified.
     * 
     * @param teamRepository
     *            the repository where the build result is found
     * @param buildResultHandle
     *            the build result to set
     * @param workItemHandle
     *            the work item to add to the build result
     * @param monitor
     *            A progress monitor, or <code>null</code> if progress
     *            monitoring is not desired.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static void linkReportedWorkItem(ITeamRepository teamRepository, IBuildResultHandle buildResultHandle,
            IWorkItemHandle workItemHandle, IProgressMonitor monitor) throws TeamRepositoryException {
        ValidationHelper.validateNotNull("teamRepository", teamRepository); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("workItemHandle", workItemHandle); //$NON-NLS-1$

        IWorkItemHandle[] reportedWorkItems = getReportedWorkItems(teamRepository, buildResultHandle, monitor);
        for (IWorkItemHandle reportedWorkItem : reportedWorkItems) {
            if (reportedWorkItem.sameItemId(workItemHandle)) {
                // Already been reported.
                return;
            }
        }

        linkWorkItem(teamRepository, buildResultHandle, workItemHandle, BuildLinkTypes.REPORTED_WORK_ITEMS, monitor);
    }
    
    
    /**
     * Adds an included work item link to the build result. This method creates a
     * link between the build result and work item. Neither object is modified.
     * 
     * @param teamRepository
     *            the repository where the build result is found
     * @param buildResultHandle
     *            the build result to set
     * @param workItemHandle
     *            the work item to add to the build result
     * @param monitor
     *            A progress monitor, or <code>null</code> if progress
     *            monitoring is not desired.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static void linkIncludedWorkItem(ITeamRepository teamRepository, IBuildResultHandle buildResultHandle,
            IWorkItemHandle workItemHandle, IProgressMonitor monitor) throws TeamRepositoryException {
        
        linkWorkItem(teamRepository, buildResultHandle, workItemHandle, BuildLinkTypes.INCLUDED_WORK_ITEMS, monitor);
    }

    /**
     * Adds a work item link to the build result of the requested type. This method creates a
     * link between the build result and work item. Neither object is modified.
     * 
     * @param teamRepository
     *            the repository where the build result is found
     * @param buildResultHandle
     *            the build result to set
     * @param workItemHandle
     *            the work item to add to the build result
     * @param linkType
     *            the BuildLinkTypes value of the type of link to create           
     * @param monitor
     *            A progress monitor, or <code>null</code> if progress
     *            monitoring is not desired.
     * @throws TeamRepositoryException
     * @throws IllegalArgumentException
     *             if any parameters are null
     * @LongOp This is a long operation; it may block indefinitely; must not be
     *         called from a responsive thread.
     */
    public static void linkWorkItem(ITeamRepository teamRepository, IBuildResultHandle buildResultHandle,
            IWorkItemHandle workItemHandle, String linkType, IProgressMonitor monitor) throws TeamRepositoryException {
        ValidationHelper.validateNotNull("teamRepository", teamRepository); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildResultHandle", buildResultHandle); //$NON-NLS-1$
        ValidationHelper.validateNotNull("workItemHandle", workItemHandle); //$NON-NLS-1$

        ILinkManager linkManager = (ILinkManager) teamRepository.getClientLibrary(ILinkManager.class);
        String commentText = getCommentText(buildResultHandle, monitor, teamRepository);

        IReference sourceReference = linkManager.referenceFactory().createReferenceToItem(buildResultHandle,
                commentText);
        IReference targetReference = linkManager.referenceFactory().createReferenceToItem(workItemHandle);
        ILink link = linkManager.createLink(linkType, sourceReference, targetReference);
        linkManager.saveLink(link, monitor);
    }

    /**
     * Get the text to set as the comment on a reference to the
     * buildResultHandle.
     * 
     * @param buildResultHandle
     *            {@link IBuildResultHandle} that reference will point to.
     * @param monitor
     *            {@link IProgressMonitor} to use when accessing the repository
     *            to get information about the buildResult.
     * @param teamRepository
     *            {@link ITeamRepository} in which the buildResultHandle
     *            resides.
     * @return generated comment text
     * @throws TeamRepositoryException
     *             if there is a problem communicating with teamRepository.
     */
    private static String getCommentText(IBuildResultHandle buildResultHandle, IProgressMonitor monitor,
            ITeamRepository teamRepository) throws TeamRepositoryException {

        ITeamBuildRecordClient buildClient = (ITeamBuildRecordClient) teamRepository.getClientLibrary(ITeamBuildRecordClient.class);
        IBuildResultRecord buildResultRecord = buildClient.getBuildResultRecord(buildResultHandle, new String[] {
                IBuildResult.PROPERTY_LABEL, IBuildResult.PROPERTY_BUILD_DEFINITION }, monitor);
        IBuildDefinition buildDefinition = buildResultRecord.getBuildDefinition();
        String comment = NLS.bind(ClientWorkItemMessages.WorkItemHelper_BUILD_LINK_LABEL, buildDefinition.getId(),
                buildResultRecord.getBuildResult().getLabel());

        IHelperType helperType = IHelperType.IRegistry.INSTANCE.getHelperType(
                LinksPackage.eINSTANCE.getReference().getName(), LinksPackage.eNS_URI);

        return ItemHelper.validateStringAttributeLength(comment, helperType,
                LinksPackage.eINSTANCE.getReference_Comment().getName());
    }
}
